@@ -27,27 +27,33 @@ in this Software without prior written authorization from The Open Group.
 
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001,2003 NoMachine, http://www.nomachine.com.           */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NX-X11, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
 /**************************************************************************/
 
+/* $XFree86: xc/lib/X11/OpenDis.c,v 3.16 2003/07/04 16:24:23 eich Exp $ */
+
 #define NEED_REPLIES
 #define NEED_EVENTS
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include "Xlibint.h"
-#include <X11/Xtrans.h>
+#include <X11/Xtrans/Xtrans.h>
 #include <X11/Xatom.h>
-#include "bigreqstr.h"
+#include <X11/Xresource.h>
+#include <X11/extensions/bigreqstr.h>
 #include <stdio.h>
 #include "Xintconn.h"
 
@@ -55,6 +61,10 @@ in this Software without prior written authorization from The Open Group.
 #include "XKBlib.h"
 #endif /* XKB */
 
+#ifdef NX_TRANS_SOCKET
+extern void *_X11TransSocketProxyConnInfo(XtransConnInfo);
+#endif
+
 #ifdef X_NOT_POSIX
 #define Size_t unsigned int
 #else
@@ -68,10 +78,6 @@ typedef struct {
     int opcode;
 } _XBigReqState;
 
-#ifdef WIN32
-int *_Xdebug_p = &_Xdebug;
-#endif
-
 #ifdef XTHREADS
 #include "locking.h"
 int  (*_XInitDisplayLock_fn)(Display *dpy) = NULL;
@@ -92,100 +98,13 @@ static void OutOfMemory(Display *dpy, char *setup);
 static Bool _XBigReqHandler(Display *dpy, xReply *rep, char *buf, int len,
 				XPointer data);
 
-#if defined(NX_CLEAN_ALLOC) || defined(NX_CLEAN_GET)  || \
-    defined(NX_CLEAN_FLUSH) || defined(NX_CLEAN_SEND) || \
-    defined(NX_CLEAN_IMAGES)
-
-/*
- * GetCleanupEnvironmentParams must be called at least once,
- * that's why we do this as soon as clients opens the display.
- * Values read from environment can (and must) be overridden
- * by agents by calling NXGetCleanupParameters().
- */
-
-static void GetEnvironmentParam(char *variable, int *flag)
-{
-    char *value;
-
-    value = getenv(variable);
-
-#ifdef NX_CLEAN_DEBUG
-    fprintf(stderr, "GetEnvironmentParam: %s is %s\n", variable, value);
-#endif
-
-    *flag = 0;
-
-    if (value != NULL && *value !='\0')
-    {
-        if (strtol(value, &value, 10) == 1)
-        {
-            *flag = 1;
-        }
-    }
-
-#ifdef NX_CLEAN_DEBUG
-    fprintf(stderr, "GetEnvironmentParam: Flag set to %d.\n", *flag);
-#endif
-}
-
-static void GetCleanupEnvironmentParams()
-{
-    /*
-     * Parse environment variables only once.
-     */
-
-    if (_NXCleanupEnvironmentCached) return;
-
-    /*
-     * Get values from environment.
-     */
-
-    if (_NXEnableCleanAlloc < 0)
-    {
-        GetEnvironmentParam("NX_CLEANALLOC", &_NXEnableCleanAlloc);
-    }
-
-    if (_NXEnableCleanGet < 0)
-    {
-        GetEnvironmentParam("NX_CLEANGET", &_NXEnableCleanGet);
-    }
-
-    if (_NXEnableCleanFlush < 0)
-    {
-        GetEnvironmentParam("NX_CLEANFLUSH", &_NXEnableCleanFlush);
-    }
-
-    if (_NXEnableCleanSend < 0)
-    {
-        GetEnvironmentParam("NX_CLEANSEND", &_NXEnableCleanSend);
-    }
-
-    if (_NXEnableCleanImages < 0)
-    {
-        GetEnvironmentParam("NX_CLEANIMAGES", &_NXEnableCleanImages);
-    }
-
-    if (_NXEnableCleanInplace < 0)
-    {
-        GetEnvironmentParam("NX_IMAGEINPLACE", &_NXEnableCleanInplace);
-    }
-
-   _NXCleanupEnvironmentCached = 1;
-}
-
-#endif /* #if defined(NX_CLEAN_ALLOC) || defined(NX_CLEAN_GET)  || ... */
-
 /* 
  * Connects to a server, creates a Display object and returns a pointer to
  * the newly created Display back to the caller.
  */
-#if NeedFunctionPrototypes
-Display *XOpenDisplay (
+Display *
+XOpenDisplay (
 	register _Xconst char *display)
-#else
-Display *XOpenDisplay (display)
-	register char *display;
-#endif
 {
 	register Display *dpy;		/* New Display object being created. */
 	register int i;
@@ -210,9 +129,12 @@ Display *XOpenDisplay (display)
 		xVisualType *vp;
 	} u;				/* proto data returned from server */
 	long setuplength;	/* number of bytes in setup message */
+	long usedbytes = 0;     /* number of bytes we have processed */
 	char *conn_auth_name, *conn_auth_data;
 	int conn_auth_namelen, conn_auth_datalen;
 	unsigned long mask;
+       long int conn_buf_size;
+       char *xlib_buffer_size;
 
 	bzero((char *) &client, sizeof(client));
 	bzero((char *) &prefix, sizeof(prefix));
@@ -220,24 +142,6 @@ Display *XOpenDisplay (display)
 #if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_TEST)
         fprintf(stderr, "\nXOpenDisplay: Called with display [%s].\n", display);
 #endif
-
-        /*
-         * Get user's settings from environment.
-         */
-
-#if defined(NX_TRANS_SOCKET)
-
-        if (_NXCleanupEnvironmentCached == 0)
-        {
-#if defined(NX_CLEAN_DEBUG)
-            fprintf(stderr, "\nXOpenDisplay: Reading cleanup parameters from environment with display [%s].\n",
-                        display);
-#endif
-            GetCleanupEnvironmentParams();
-        }
-
-#endif
-
 	/*
 	 * If the display specifier string supplied as an argument to this 
 	 * routine is NULL or a pointer to NULL, read the DISPLAY variable.
@@ -252,7 +156,6 @@ Display *XOpenDisplay (display)
 		/* Display is non-NULL, copy the pointer */
 		display_name = (char *)display;
 	}
-
 /*
  * Set the default error handlers.  This allows the global variables to
  * default to NULL for use with shared libraries.
@@ -285,7 +188,7 @@ Display *XOpenDisplay (display)
 	dpy->fd = _X11TransGetConnectionNumber (dpy->trans_conn);
 
 #if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_TEST) 
-       fprintf(stderr, "\nXOpenDisplay: Connected display with dpy->fd = [%d].\n", dpy->fd);
+        fprintf(stderr, "\nXOpenDisplay: Connected display with dpy->fd = [%d].\n", dpy->fd);
 #endif
 	/* Initialize as much of the display structure as we can.
 	 * Initialize pointers to NULL so that XFreeDisplayStructure will
@@ -371,11 +274,28 @@ Display *XOpenDisplay (display)
 	}	
 
 	/* Set up the output buffers. */
-	if ((dpy->bufptr = dpy->buffer = Xcalloc(1, BUFSIZE)) == NULL) {
-	        OutOfMemory (dpy, setup);
-		return(NULL);
-	}
-	dpy->bufmax = dpy->buffer + BUFSIZE;
+#ifndef XLIBDEFAULTBUFSIZE
+#define XLIBDEFAULTBUFSIZE 16384 /* 16k */
+#endif
+#ifndef XLIBMINBUFSIZE
+#define XLIBMINBUFSIZE BUFSIZE /* old default buffer size */
+#endif
+    if ((xlib_buffer_size = getenv("XLIBBUFFERSIZE")) == NULL)
+        conn_buf_size = XLIBDEFAULTBUFSIZE;
+    else
+        conn_buf_size = 1024 * strtol(xlib_buffer_size, NULL, 10);
+    if (conn_buf_size < XLIBMINBUFSIZE)
+        conn_buf_size = XLIBMINBUFSIZE;
+#if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_TEST) 
+    fprintf (stderr, "Xlib: Running with XLIBBUFFERSIZE [%d] XLIBMINBUFSIZE [%d] "
+                 "buffer size [%ld].\n", XLIBDEFAULTBUFSIZE, XLIBMINBUFSIZE, conn_buf_size);
+#endif
+
+    if ((dpy->bufptr = dpy->buffer = Xcalloc(1, conn_buf_size)) == NULL) {
+         OutOfMemory (dpy, setup);
+         return(NULL);
+    }
+    dpy->bufmax = dpy->buffer + conn_buf_size;
  
 	/* Set up the input event queue and input event queue parameters. */
 	dpy->head = dpy->tail = NULL;
@@ -436,9 +356,16 @@ Display *XOpenDisplay (display)
 
 	if (prefix.majorVersion != X_PROTOCOL) {
 	    /* XXX - printing messages marks a bad programming interface */
+#ifdef NX_TRANS_SOCKET
+            if (_X11TransSocketProxyConnInfo(dpy->trans_conn) == NULL) {
+                fprintf (stderr, "Xlib: client uses different protocol version (%d) "
+                             "than server (%d)!\r\n", X_PROTOCOL, prefix.majorVersion);
+            }
+#else
 	    fprintf (stderr,
      "Xlib: client uses different protocol version (%d) than server (%d)!\r\n",
 		     X_PROTOCOL, prefix.majorVersion);
+#endif
 	    _XDisconnectDisplay (dpy->trans_conn);
 	    Xfree ((char *)dpy);
 	    return(NULL);
@@ -452,6 +379,7 @@ Display *XOpenDisplay (display)
 		return(NULL);
 	}
 	_XRead (dpy, (char *)u.setup, setuplength);
+
 /*
  * If the connection was not accepted by the server due to problems,
  * give error message to the user....
@@ -461,13 +389,29 @@ Display *XOpenDisplay (display)
 		fprintf (stderr, 
 		      "Xlib: connection to \"%s\" refused by server\r\nXlib: ",
 			 fullname);
-		(void) fwrite (u.failure, (Size_t)sizeof(char),
+
+		if (prefix.lengthReason > setuplength) {
+		    fprintf (stderr, "Xlib: Broken initial reply: length of reason > length of packet\r\n");
+		}else{
+		    (void) fwrite (u.failure, (Size_t)sizeof(char),
 			       (Size_t)prefix.lengthReason, stderr);
-		(void) fwrite ("\r\n", sizeof(char), 2, stderr);
+		    (void) fwrite ("\r\n", sizeof(char), 2, stderr);
+		}
+
 		OutOfMemory(dpy, setup);
 		return (NULL);
 	}
 
+/*
+ * Check if the reply was long enough to get any information out of it.
+ */
+	usedbytes = sz_xConnSetup;
+	if (setuplength < usedbytes ) {
+	    fprintf (stderr, "Xlib: Broken initial reply: Too short (%ld)\n", setuplength);
+	    OutOfMemory(dpy, setup);
+	    return (NULL);
+	}
+
 /*
  * We succeeded at authorization, so let us move the data into
  * the display structure.
@@ -511,23 +455,30 @@ Display *XOpenDisplay (display)
 	    OutOfMemory(dpy, setup);
 	    return (NULL);
 	}
+
 	dpy->vendor = (char *) Xmalloc((unsigned) (u.setup->nbytesVendor + 1));
 	if (dpy->vendor == NULL) {
 	    OutOfMemory(dpy, setup);
 	    return (NULL);
 	}
 	vendorlen = u.setup->nbytesVendor;
+
+/*
+ * validate setup length
+ */
+	usedbytes += (vendorlen + 3) & ~3;
+	if (setuplength < usedbytes) {
+	    fprintf (stderr, "Xlib: Broken initial reply: Too short (%ld)\n", setuplength);
+	    OutOfMemory(dpy, setup);
+	    return (NULL);
+	}
+
  	u.setup = (xConnSetup *) (((char *) u.setup) + sz_xConnSetup);
   	(void) strncpy(dpy->vendor, u.vendor, vendorlen);
 	dpy->vendor[vendorlen] = '\0';
  	vendorlen = (vendorlen + 3) & ~3;	/* round up */
-/*
- * validate setup length
- */
-	if ((int) setuplength - sz_xConnSetup - vendorlen < 0) {
-	    OutOfMemory(dpy, setup);
-	    return (NULL);
-	}
+
+
 	memmove (setup, u.vendor + vendorlen,
 		 (int) setuplength - sz_xConnSetup - vendorlen);
  	u.vendor = setup;
@@ -544,6 +495,14 @@ Display *XOpenDisplay (display)
 /*
  * First decode the Z axis Screen format information.
  */
+	usedbytes += dpy->nformats * sz_xPixmapFormat;
+
+	if (setuplength < usedbytes) {
+	    fprintf (stderr, "Xlib: Broken initial reply: Too short (%ld)\n", setuplength);
+	    OutOfMemory (dpy, setup);
+	    return(NULL);
+	}
+
 	for (i = 0; i < dpy->nformats; i++) {
 	    register ScreenFormat *fmt = &dpy->pixmap_format[i];
 	    fmt->depth = u.sf->depth;
@@ -562,12 +521,22 @@ Display *XOpenDisplay (display)
 	        OutOfMemory (dpy, setup);
 		return(NULL);
 	}
+
 /*
  * Now go deal with each screen structure.
  */
 	for (i = 0; i < dpy->nscreens; i++) {
 	    register Screen *sp = &dpy->screens[i];
-	    VisualID root_visualID = u.rp->rootVisualID;
+	    VisualID root_visualID;
+
+	    usedbytes += sz_xWindowRoot;
+	    if (setuplength < usedbytes) {
+		fprintf (stderr, "Xlib: Broken initial reply: Too short (%ld)\n", setuplength);
+		OutOfMemory (dpy, setup);
+		return(NULL);
+	    }
+
+	    root_visualID = u.rp->rootVisualID;	    
 	    sp->display	    = dpy;
 	    sp->root 	    = u.rp->windowId;
 	    sp->cmap 	    = u.rp->defaultColormap;
@@ -600,6 +569,14 @@ Display *XOpenDisplay (display)
 	     */
 	    for (j = 0; j < sp->ndepths; j++) {
 		Depth *dp = &sp->depths[j];
+
+		usedbytes += sz_xDepth;
+		if (setuplength < usedbytes) {
+		    fprintf (stderr, "Xlib: Broken initial reply: Too short (%ld)\n", setuplength);
+		    OutOfMemory (dpy, setup);
+		    return(NULL);
+		}
+		
 		dp->depth = u.dp->depth;
 		dp->nvisuals = u.dp->nVisuals;
 		u.dp = (xDepth *) (((char *) u.dp) + sz_xDepth);
@@ -612,6 +589,14 @@ Display *XOpenDisplay (display)
 		    }
 		    for (k = 0; k < dp->nvisuals; k++) {
 			register Visual *vp = &dp->visuals[k];
+
+			usedbytes += sz_xVisualType;
+			if (setuplength < usedbytes) {
+			    fprintf (stderr, "Xlib: Broken initial reply: Too short (%ld)\n", setuplength);
+			    OutOfMemory (dpy, setup);
+			    return(NULL);
+			}
+			
 			vp->visualid	= u.vp->visualID;
 			vp->class	= u.vp->class;
 			vp->bits_per_rgb= u.vp->bitsPerRGB;
@@ -623,13 +608,27 @@ Display *XOpenDisplay (display)
 			u.vp = (xVisualType *) (((char *) u.vp) +
 						sz_xVisualType);
 		    }
+		    if (dp->depth == 32 && getenv ("XLIB_SKIP_ARGB_VISUALS"))
+		    {
+			Xfree (dp->visuals);
+			dp->visuals = NULL;
+			dp->nvisuals = 0;
+		    }
 		} else {
 		    dp->visuals = (Visual *) NULL;
 		}
 	    }
 	    sp->root_visual = _XVIDtoVisual(dpy, root_visualID);
 	}
-		
+
+	if(usedbytes != setuplength){
+	    /* Sanity check, shouldn't happen. */
+	    fprintf(stderr, "Xlib: Did not parse entire setup message: "
+	                    "parsed: %ld, message: %ld\n",
+		    usedbytes, setuplength);
+	    OutOfMemory(dpy, setup);
+	    return(NULL);
+	}
 
 /*
  * Now start talking to the server to setup all other information...
@@ -738,6 +737,9 @@ Display *XOpenDisplay (display)
 /*
  * and return successfully
  */
+#if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_TEST) 
+        fprintf(stderr, "XOpenDisplay: Returning display at [%p].\n", dpy);
+#endif
  	return(dpy);
 }
 
@@ -806,6 +808,10 @@ void _XFreeDisplayStructure(dpy)
 	if (dpy->xkb_info)
 	   (*dpy->free_funcs->xkb)(dpy);
 
+	/* if RM database was allocated by XGetDefault() free it */
+	if (dpy->db && (dpy->flags & XlibDisplayDfltRMDB))
+	    XrmDestroyDatabase(dpy->db);
+
 	if (dpy->screens) {
 	    register int i;
 
