@@ -30,21 +30,23 @@ from The Open Group.
 
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001,2003 NoMachine, http://www.nomachine.com.           */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NX-X11, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
 /**************************************************************************/
 
+/* $XFree86: xc/lib/X11/Xlibint.h,v 3.27 2003/05/27 22:26:26 tsi Exp $ */
+
 #ifndef _XLIBINT_H_
 #define _XLIBINT_H_ 1
 
@@ -58,70 +60,14 @@ from The Open Group.
 
 #include <X11/Xlib.h>
 #include <X11/Xproto.h>		/* to declare xEvent */
-
-#if defined(NX_CLEAN_ALLOC) || defined(NX_CLEAN_GET)  || \
-    defined(NX_CLEAN_FLUSH) || defined(NX_CLEAN_SEND) || \
-    defined(NX_CLEAN_IMAGES)
-
-typedef struct {
-    int bitmap_rect_width;
-    int bitmap_rect_height;
-    int pixmap_rect_width;
-    int pixmap_rect_height;
-} SplitParams;
-
-typedef struct {
-    unsigned int color_mask;
-    unsigned int correction_mask;
-    unsigned int white_threshold;
-    unsigned int black_threshold;
-} ColorParams;
-
-/*
- * In XlibInt.c and PutImage.c.
- */
-
-extern int _NXCleanupEnvironmentCached;
-extern int _NXImageEnvironmentCached;
-
-extern int _NXEnableCleanAlloc;
-extern int _NXEnableCleanGet;
-extern int _NXEnableCleanFlush;
-extern int _NXEnableCleanSend;
-extern int _NXEnableCleanImages;
-extern int _NXEnableCleanInplace;
-
-extern int _NXEnableImageSplit;
-extern int _NXEnableImageMask;
-extern int _NXEnableImageFrame;
-
-extern SplitParams _NXSplitParams;
-extern ColorParams _NXColorParams;
-
-#endif
+#include <X11/XlibConf.h>	/* for configured options like XTHREADS */
 
 #ifdef NX_TRANS_SOCKET
 
-/*
- * Original output buffer size in stock XFree86 is
- * 2048. We try to reduce context switches and help
- * stream compression by increasing the maximum size
- * of the buffer 8192. _NXFlushSize determines when
- * the display buffer is actually flushed. It is set
- * by default to 4096 but agents should set it to 0
- * if they want to disable early flush.
- */
+#include "NXvars.h"
 
-extern int _NXFlushSize;
-
-/*
- * Let user redefine function printing an error
- * message in the case of out-of-order sequence
- * numbers.
- */
-
-extern void (*_NXLostSequenceFunction)(Display *dpy, unsigned long newseq,
-                                           unsigned long lastseq, unsigned int type);
+#define _XGetIOError(dpy) \
+    (dpy -> flags & XlibDisplayIOError)
 
 #endif
 
@@ -161,9 +107,7 @@ struct _XDisplay
 	XID resource_id;	/* allocator current ID */
 	int resource_shift;	/* allocator shift to correct bits */
 	XID (*resource_alloc)(	/* allocator function */
-#if NeedFunctionPrototypes
 		struct _XDisplay*
-#endif
 		);
 	int byte_order;		/* screen byte order, LSBFirst, MSBFirst */
 	int bitmap_unit;	/* padding and data requirements */
@@ -184,9 +128,7 @@ struct _XDisplay
 	unsigned max_request_size; /* maximum number 32 bit words in request*/
 	struct _XrmHashBucketRec *db;
 	int (*synchandler)(	/* Synchronization handler */
-#if NeedFunctionPrototypes
 		struct _XDisplay*
-#endif
 		);
 	char *display_name;	/* "host:display" string used on this connect*/
 	int default_screen;	/* default screen for operations */
@@ -296,10 +238,8 @@ typedef struct _XSQEvent
 #define NEED_REPLIES
 #endif
 
-#if NeedFunctionPrototypes	/* prototypes require event type definitions */
 #define NEED_EVENTS
 #define NEED_REPLIES
-#endif
 #include <X11/Xproto.h>
 #ifdef __sgi
 #define _SGI_MP_SOURCE  /* turn this on to get MP safe errno */
@@ -362,32 +302,24 @@ struct _XLockPtrs {
 
 /* in XlibInt.c */
 extern void (*_XCreateMutex_fn)(
-#if NeedFunctionPrototypes
     LockInfoPtr /* lock */
-#endif
 );
 extern void (*_XFreeMutex_fn)(
-#if NeedFunctionPrototypes
     LockInfoPtr /* lock */
-#endif
 );
 extern void (*_XLockMutex_fn)(
-#if NeedFunctionPrototypes
     LockInfoPtr	/* lock */
 #if defined(XTHREADS_WARN) || defined(XTHREADS_FILE_LINE)
     , char * /* file */
     , int /* line */
 #endif
-#endif
 );
 extern void (*_XUnlockMutex_fn)(
-#if NeedFunctionPrototypes
     LockInfoPtr	/* lock */
 #if defined(XTHREADS_WARN) || defined(XTHREADS_FILE_LINE)
     , char * /* file */
     , int /* line */
 #endif
-#endif
 );
 
 extern LockInfoPtr _Xglobal_lock;
@@ -443,28 +375,15 @@ extern LockInfoPtr _Xglobal_lock;
 #define LOCKED 1
 #define UNLOCKED 0
 
-/*
- * Set display buffer to 8192 instead of 2048. What we
- * want is to reduce the context switches and increase
- * the compression performances. Value of _NXFlushSize
- * will determine when the display buffer is actually
- * flushed.
- */
-
 #ifdef NX_TRANS_SOCKET
-
-#ifndef BUFSIZE
-#define BUFSIZE 8192			/* X output buffer size. */
+#ifndef BUFSIZE                         /* Output buffer size is configurable  */
+#define BUFSIZE 8192                    /* but this is still used for reading. */
 #endif
-
 #else
-
 #ifndef BUFSIZE
 #define BUFSIZE 2048			/* X output buffer size. */
 #endif
-
 #endif
-
 #ifndef PTSPERBATCH
 #define PTSPERBATCH 1024		/* point batching */
 #endif
@@ -501,6 +420,7 @@ extern LockInfoPtr _Xglobal_lock;
 #define XlibDisplayReadEvents	(1L << 5) /* in _XReadEvents */
 #define XlibDisplayReply	(1L << 5) /* in _XReply */
 #define XlibDisplayWriting	(1L << 6) /* in _XFlushInt, _XSend */
+#define XlibDisplayDfltRMDB     (1L << 7) /* mark if RM db from XGetDefault */
 
 /*
  * X Protocol packetizing macros.
@@ -536,31 +456,15 @@ extern LockInfoPtr _Xglobal_lock;
  *
  */
 
-#ifdef NX_CLEAN_GET
-#define _NXCleanGetReq(dpy, req, size) \
-if (_NXEnableCleanGet > 0) \
-{ \
-    while (dpy->bufptr - (char *) req < size) \
-        *(dpy->bufptr++) = 0; \
-} \
-else \
-{ \
-    dpy->bufptr += size; \
-}
-#else
-#define _NXCleanGetReq(dpy, req, size) \
-            dpy->bufptr += size;
-#endif
-
 #if !defined(UNIXCPP) || defined(ANSICPP)
 #define GetReq(name, req) \
         WORD64ALIGN\
 	if ((dpy->bufptr + SIZEOF(x##name##Req)) > dpy->bufmax)\
 		_XFlush(dpy);\
 	req = (x##name##Req *)(dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, (SIZEOF(x##name##Req)));\
 	req->reqType = X_##name;\
 	req->length = (SIZEOF(x##name##Req))>>2;\
+	dpy->bufptr += SIZEOF(x##name##Req);\
 	dpy->request++
 
 #else  /* non-ANSI C uses empty comment instead of "##" for token concatenation */
@@ -569,9 +473,9 @@ else \
 	if ((dpy->bufptr + SIZEOF(x/**/name/**/Req)) > dpy->bufmax)\
 		_XFlush(dpy);\
 	req = (x/**/name/**/Req *)(dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, (SIZEOF(x/**/name/**/Req)));\
 	req->reqType = X_/**/name;\
 	req->length = (SIZEOF(x/**/name/**/Req))>>2;\
+	dpy->bufptr += SIZEOF(x/**/name/**/Req);\
 	dpy->request++
 #endif
 
@@ -584,9 +488,9 @@ else \
 	if ((dpy->bufptr + SIZEOF(x##name##Req) + n) > dpy->bufmax)\
 		_XFlush(dpy);\
 	req = (x##name##Req *)(dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, (SIZEOF(x##name##Req) + n));\
 	req->reqType = X_##name;\
 	req->length = (SIZEOF(x##name##Req) + n)>>2;\
+	dpy->bufptr += SIZEOF(x##name##Req) + n;\
 	dpy->request++
 #else
 #define GetReqExtra(name, n, req) \
@@ -594,7 +498,6 @@ else \
 	if ((dpy->bufptr + SIZEOF(x/**/name/**/Req) + n) > dpy->bufmax)\
 		_XFlush(dpy);\
 	req = (x/**/name/**/Req *)(dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, (SIZEOF(x/**/name/**/Req) + n));\
 	req->reqType = X_/**/name;\
 	req->length = (SIZEOF(x/**/name/**/Req) + n)>>2;\
 	dpy->bufptr += SIZEOF(x/**/name/**/Req) + n;\
@@ -614,10 +517,10 @@ else \
 	if ((dpy->bufptr + SIZEOF(xResourceReq)) > dpy->bufmax)\
 	    _XFlush(dpy);\
 	req = (xResourceReq *) (dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, (SIZEOF(xResourceReq)));\
 	req->reqType = X_##name;\
 	req->length = 2;\
 	req->id = (rid);\
+	dpy->bufptr += SIZEOF(xResourceReq);\
 	dpy->request++
 #else
 #define GetResReq(name, rid, req) \
@@ -625,10 +528,10 @@ else \
 	if ((dpy->bufptr + SIZEOF(xResourceReq)) > dpy->bufmax)\
 	    _XFlush(dpy);\
 	req = (xResourceReq *) (dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, (SIZEOF(xResourceReq)));\
 	req->reqType = X_/**/name;\
 	req->length = 2;\
 	req->id = (rid);\
+	dpy->bufptr += SIZEOF(xResourceReq);\
 	dpy->request++
 #endif
 
@@ -642,9 +545,9 @@ else \
 	if ((dpy->bufptr + SIZEOF(xReq)) > dpy->bufmax)\
 	    _XFlush(dpy);\
 	req = (xReq *) (dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, SIZEOF(xReq));\
 	req->reqType = X_##name;\
 	req->length = 1;\
+	dpy->bufptr += SIZEOF(xReq);\
 	dpy->request++
 #else
 #define GetEmptyReq(name, req) \
@@ -652,9 +555,9 @@ else \
 	if ((dpy->bufptr + SIZEOF(xReq)) > dpy->bufmax)\
 	    _XFlush(dpy);\
 	req = (xReq *) (dpy->last_req = dpy->bufptr);\
-	_NXCleanGetReq(dpy, req, SIZEOF(xReq));\
 	req->reqType = X_/**/name;\
 	req->length = 1;\
+	dpy->bufptr += SIZEOF(xReq);\
 	dpy->request++
 #endif
 
@@ -745,21 +648,11 @@ extern void _XFlushGCCache(Display *dpy, GC gc);
  *    BufAlloc (xTextElt *, elt, nbytes)
  */
 
-#ifdef NX_CLEAN_ALLOC
-#define _NXCleanBufAlloc(dpy, n) \
-if (_NXEnableCleanAlloc > 0) \
-{ \
-    memset((char *) dpy->bufptr, 0, n); \
-}
-#else
-#define _NXCleanBufAlloc(dpy, n) ;
-#endif
-
 #define BufAlloc(type, ptr, n) \
     if (dpy->bufptr + (n) > dpy->bufmax) \
         _XFlush (dpy); \
     ptr = (type) dpy->bufptr; \
-    _NXCleanBufAlloc(dpy, (n));\
+    (void)ptr; \
     dpy->bufptr += (n);
 
 #ifdef WORD64
@@ -772,18 +665,14 @@ if (_NXEnableCleanAlloc > 0) \
 #ifdef LONG64
 #define Data32(dpy, data, len) _XData32(dpy, (long *)data, len)
 extern int _XData32(
-#if NeedFunctionPrototypes
 	     Display *dpy,
 	     register long *data,
 	     unsigned len
-#endif
 );
 extern void _XRead32(
-#if NeedFunctionPrototypes
 	     Display *dpy,
 	     register long *data,
 	     long len
-#endif
 );
 #else
 #define Data32(dpy, data, len) Data((dpy), (char *)(data), (len))
@@ -884,13 +773,11 @@ typedef struct _XInternalAsync {
      * The handler returns True iff it handled this reply.
      */
     Bool (*handler)(
-#if NeedNestedPrototypes
 		    Display*	/* dpy */,
 		    xReply*	/* rep */,
 		    char*	/* buf */,
 		    int		/* len */,
 		    XPointer	/* data */
-#endif
 		    );
     XPointer data;
 } _XAsyncHandler;
@@ -914,15 +801,11 @@ extern void _XDeqAsyncHandler(Display *dpy, _XAsyncHandler *handler);
     }
 
 typedef void (*FreeFuncType) (
-#if NeedFunctionPrototypes
     Display*	/* display */
-#endif
 );
 
 typedef int (*FreeModmapType) (
-#if NeedFunctionPrototypes
     XModifierKeymap*	/* modmap */
-#endif
 );
 
 /*
@@ -942,94 +825,72 @@ typedef struct _XFreeFuncs {
 
 /* types for InitExt.c */
 typedef int (*CreateGCType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     GC		/* gc */,
     XExtCodes*	/* codes */
-#endif
 );
 
 typedef int (*CopyGCType)(
-#if NeedFunctionPrototypes
     Display*	/* display */,
     GC		/* gc */,
     XExtCodes*	/* codes */
-#endif
 );
 
 typedef int (*FlushGCType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     GC		/* gc */,
     XExtCodes*	/* codes */
-#endif
 );
 
 typedef int (*FreeGCType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     GC		/* gc */,
     XExtCodes*	/* codes */
-#endif
 );
 
 typedef int (*CreateFontType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     XFontStruct* /* fs */,
     XExtCodes*	/* codes */
-#endif
 );
 
 typedef int (*FreeFontType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     XFontStruct* /* fs */,
     XExtCodes*	/* codes */
-#endif
 );
 
 typedef int (*CloseDisplayType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     XExtCodes*	/* codes */
-#endif
 );
 
 typedef int (*ErrorType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     xError*	/* err */,
     XExtCodes*	/* codes */,
     int*	/* ret_code */
-#endif
 );
 
 typedef char* (*ErrorStringType) (
-#if NeedFunctionPrototypes
     Display*	/* display */,
     int		/* code */,
     XExtCodes*	/* codes */,
     char*	/* buffer */,
     int		/* nbytes */
-#endif
 );
 
 typedef void (*PrintErrorType)(
-#if NeedFunctionPrototypes
     Display*	/* display */,
     XErrorEvent* /* ev */,
     void*	/* fp */
-#endif
 );
 
 typedef void (*BeforeFlushType)(
-#if NeedFunctionPrototypes
     Display*	/* display */,
     XExtCodes*	/* codes */,
     _Xconst char* /* data */,
     long	/* len */
-#endif
 );
 
 /*
@@ -1059,87 +920,60 @@ typedef struct _XExten {		/* private to extension mechanism */
 extern void Data(Display *dpy, char *data, long len);
 #endif
 extern int _XError(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     xError*	/* rep */
-#endif
 );
 extern int _XIOError(
-#if NeedFunctionPrototypes
     Display*	/* dpy */
-#endif
 );
 extern int (*_XIOErrorFunction)(
-#if NeedNestedPrototypes
     Display*	/* dpy */
-#endif
 );
 extern int (*_XErrorFunction)(
-#if NeedNestedPrototypes
     Display*		/* dpy */,
     XErrorEvent*	/* error_event */
-#endif
 );
 extern void _XEatData(
-#if NeedFunctionPrototypes
     Display*		/* dpy */,
     unsigned long	/* n */
-#endif
 );
 extern char *_XAllocScratch(
-#if NeedFunctionPrototypes
     Display*		/* dpy */,
     unsigned long	/* nbytes */
-#endif
 );
 extern char *_XAllocTemp(
-#if NeedFunctionPrototypes
     Display*		/* dpy */,
     unsigned long	/* nbytes */
-#endif
 );
 extern void _XFreeTemp(
-#if NeedFunctionPrototypes
     Display*		/* dpy */,
     char*		/* buf */,
     unsigned long	/* nbytes */
-#endif
 );
 extern Visual *_XVIDtoVisual(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     VisualID	/* id */
-#endif
 );
 extern unsigned long _XSetLastRequestRead(
-#if NeedFunctionPrototypes
     Display*		/* dpy */,
     xGenericReply*	/* rep */
-#endif
 );
 extern int _XGetHostname(
-#if NeedFunctionPrototypes
     char*	/* buf */,
     int		/* maxlen */
-#endif
 );
 extern Screen *_XScreenOfWindow(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     Window	/* w */
-#endif
 );
 extern Bool _XAsyncErrorHandler(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     xReply*	/* rep */,
     char*	/* buf */,
     int		/* len */,
     XPointer	/* data */
-#endif
 );
 extern char *_XGetAsyncReply(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     char*	/* replbuf */,
     xReply*	/* rep */,
@@ -1147,10 +981,8 @@ extern char *_XGetAsyncReply(
     int		/* len */,
     int		/* extra */,
     Bool	/* discard */
-#endif
 );
 extern void _XGetAsyncData(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     char *	/* data */,
     char *	/* buf */,
@@ -1158,374 +990,261 @@ extern void _XGetAsyncData(
     int		/* skip */,
     int		/* datalen */,
     int		/* discardtotal */
-#endif
 );
 extern void _XFlush(
-#if NeedFunctionPrototypes
     Display*	/* dpy */
-#endif
 );
 extern int _XEventsQueued(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     int 	/* mode */
-#endif
 );
 extern void _XReadEvents(
-#if NeedFunctionPrototypes
     Display*	/* dpy */
-#endif
 );
 extern int _XRead(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     char*	/* data */,
     long	/* size */
-#endif
 );
 extern void _XReadPad(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     char*	/* data */,
     long	/* size */
-#endif
 );
 extern void _XSend(
-#if NeedFunctionPrototypes
     Display*		/* dpy */,
     _Xconst char*	/* data */,
     long		/* size */
-#endif
 );
 extern Status _XReply(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     xReply*	/* rep */,
     int		/* extra */,
     Bool	/* discard */
-#endif
 );
 extern void _XEnq(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     xEvent*	/* event */
-#endif
 );
 extern void _XDeq(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     _XQEvent*	/* prev */,
     _XQEvent*	/* qelt */
-#endif
 );
 
 extern Bool _XUnknownWireEvent(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     XEvent*	/* re */,
     xEvent*	/* event */
-#endif
 );
 extern Status _XUnknownNativeEvent(
-#if NeedFunctionPrototypes
     Display*	/* dpy */,
     XEvent*	/* re */,
     xEvent*	/* event */
-#endif
 );
 
 extern Bool _XWireToEvent(Display *dpy, XEvent *re, xEvent *event);
 extern Bool _XDefaultWireError(Display *display, XErrorEvent *he, xError *we);
 extern Bool _XPollfdCacheInit(Display *dpy);
+extern void _XPollfdCacheAdd(Display *dpy, int fd);
+extern void _XPollfdCacheDel(Display *dpy, int fd);
 extern XID _XAllocID(Display *dpy);
 extern void _XAllocIDs(Display *dpy, XID *ids, int count);
 
 extern int _XFreeExtData(
-#if NeedFunctionPrototypes
     XExtData*	/* extension */
-#endif
 );
 
 extern int (*XESetCreateGC(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
 	      GC			/* gc */,
 	      XExtCodes*		/* codes */
-#endif
 	    )		/* proc */
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, GC, XExtCodes*
-#endif
 );
 
 extern int (*XESetCopyGC(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               GC			/* gc */,
               XExtCodes*		/* codes */
-#endif
             )		/* proc */	      
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, GC, XExtCodes*
-#endif
 );
 
 extern int (*XESetFlushGC(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               GC			/* gc */,
               XExtCodes*		/* codes */
-#endif
             )		/* proc */	     
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, GC, XExtCodes*
-#endif
 );
 
 extern int (*XESetFreeGC(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               GC			/* gc */,
               XExtCodes*		/* codes */
-#endif
             )		/* proc */	     
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, GC, XExtCodes*
-#endif
 );
 
 extern int (*XESetCreateFont(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               XFontStruct*		/* fs */,
               XExtCodes*		/* codes */
-#endif
             )		/* proc */    
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XFontStruct*, XExtCodes*
-#endif
 );
 
 extern int (*XESetFreeFont(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               XFontStruct*		/* fs */,
               XExtCodes*		/* codes */
-#endif
             )		/* proc */    
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XFontStruct*, XExtCodes*
-#endif
 ); 
 
 extern int (*XESetCloseDisplay(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               XExtCodes*		/* codes */
-#endif
             )		/* proc */    
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XExtCodes*
-#endif
 );
 
 extern int (*XESetError(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     int (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               xError*			/* err */,
               XExtCodes*		/* codes */,
               int*			/* ret_code */
-#endif
             )		/* proc */    
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, xError*, XExtCodes*, int*
-#endif
 );
 
 extern char* (*XESetErrorString(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     char* (*) (
-#if NeedNestedPrototypes
 	        Display*		/* display */,
                 int			/* code */,
                 XExtCodes*		/* codes */,
                 char*			/* buffer */,
                 int			/* nbytes */
-#endif
               )		/* proc */	       
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, int, XExtCodes*, char*, int
-#endif
 );
 
 extern void (*XESetPrintErrorValues (
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* extension */,
     void (*)(
-#if NeedNestedPrototypes
 	      Display*			/* display */,
 	      XErrorEvent*		/* ev */,
 	      void*			/* fp */
-#endif
 	     )		/* proc */
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XErrorEvent*, void*
-#endif
 );
 
 extern Bool (*XESetWireToEvent(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* event_number */,
     Bool (*) (
-#if NeedNestedPrototypes
 	       Display*			/* display */,
                XEvent*			/* re */,
                xEvent*			/* event */
-#endif
              )		/* proc */    
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XEvent*, xEvent*
-#endif
 );
 
 extern Status (*XESetEventToWire(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* event_number */,
     Status (*) (
-#if NeedNestedPrototypes
 	      Display*			/* display */,
               XEvent*			/* re */,
               xEvent*			/* event */
-#endif
             )		/* proc */   
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XEvent*, xEvent*
-#endif
 );
 
 extern Bool (*XESetWireToError(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* error_number */,
     Bool (*) (
-#if NeedNestedPrototypes
 	       Display*			/* display */,
 	       XErrorEvent*		/* he */,
 	       xError*			/* we */
-#endif
             )		/* proc */   
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XErrorEvent*, xError*
-#endif
 );
 
 extern void (*XESetBeforeFlush(
-#if NeedFunctionPrototypes
     Display*		/* display */,
     int			/* error_number */,
     void (*) (
-#if NeedNestedPrototypes
 	       Display*			/* display */,
 	       XExtCodes*		/* codes */,
 	       _Xconst char*		/* data */,
 	       long			/* len */
-#endif
             )		/* proc */   
-#endif
 ))(
-#if NeedNestedPrototypes
     Display*, XExtCodes*, _Xconst char*, long
-#endif
 );
 
 /* internal connections for IMs */
 
 typedef void (*_XInternalConnectionProc)(
-#if NeedFunctionPrototypes
     Display*			/* dpy */,
     int				/* fd */,
     XPointer			/* call_data */
-#endif
 );
 
 
 extern Status _XRegisterInternalConnection(
-#if NeedFunctionPrototypes
     Display*			/* dpy */,
     int				/* fd */,
     _XInternalConnectionProc	/* callback */,
     XPointer			/* call_data */
-#endif
 );
 
 extern void _XUnregisterInternalConnection(
-#if NeedFunctionPrototypes
     Display*			/* dpy */,
     int				/* fd */
-#endif
 );
 
 /* Display structure has pointers to these */
@@ -1546,48 +1265,36 @@ struct _XConnWatchInfo {	/* info from XAddConnectionWatch */
 
 #ifdef __UNIXOS2__
 extern char* __XOS2RedirRoot(
-#if NeedFunctionPrototypes
     char*
-#endif
 );
 #endif
 
 extern int _XTextHeight(
-#if NeedFunctionPrototypes
     XFontStruct*	/* font_struct */,
     _Xconst char*	/* string */,
     int			/* count */
-#endif
 );
 
 extern int _XTextHeight16(
-#if NeedFunctionPrototypes
     XFontStruct*	/* font_struct */,
     _Xconst XChar2b*	/* string */,
     int			/* count */
-#endif
 );
 
 #if defined(WIN32)
 
 extern int _XOpenFile(
-#if NeedFunctionPrototypes
     _Xconst char*	/* path */,
     int			/* flags */
-#endif
 );
 
 extern void* _XFopenFile(
-#if NeedFunctionPrototypes
     _Xconst char*	/* path */,
     _Xconst char*	/* mode */
-#endif
 );
 
 extern int _XAccessFile(
-#if NeedFunctionPrototypes
     _Xconst char*	/* path */
-#endif
 );
 #else
 #define _XOpenFile(path,flags) open(path,flags)
@@ -1598,14 +1305,42 @@ extern int _XAccessFile(
 extern Status _XEventToWire(Display *dpy, XEvent *re, xEvent *event);
 
 extern int _XF86LoadQueryLocaleFont(
-#if NeedFunctionPrototypes
     Display*		/* dpy */,
     _Xconst char*	/* name*/,
     XFontStruct**	/* xfp*/,
     Font*		/* fidp */
-#endif
 );
 
+extern void _XProcessWindowAttributes (
+    register Display *dpy,
+    xChangeWindowAttributesReq *req,
+    register unsigned long valuemask,
+    register XSetWindowAttributes *attributes);
+
+extern int _XDefaultError(
+        Display *dpy,
+        XErrorEvent *event);
+
+extern int _XDefaultIOError(
+        Display *dpy);
+
+extern void _XSetClipRectangles (
+    register Display *dpy,
+    GC gc,
+    int clip_x_origin, int clip_y_origin,
+    XRectangle *rectangles,
+    int n,
+    int ordering);
+
+Status _XGetWindowAttributes(
+    register Display *dpy,
+    Window w,
+    XWindowAttributes *attr);
+
+int _XPutBackEvent (
+    register Display *dpy, 
+    register XEvent *event);
+
 _XFUNCPROTOEND
 
 #endif /* _XLIBINT_H_ */
