@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/os/WaitFor.c,v 3.38 2002/05/31 18:46:05 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/os/WaitFor.c,v 3.42 2003/10/16 01:33:35 dawes Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -50,16 +50,16 @@ SOFTWARE.
 
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001,2003 NoMachine, http://www.nomachine.com.           */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NX-X11, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -73,19 +73,17 @@ SOFTWARE.
  *
  *****************************************************************/
 
-#if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_WAKEUP)
-
-static unsigned long startTimeInMillis;
-
+#ifdef HAVE_DIX_CONFIG_H
+#include <dix-config.h>
 #endif
 
 #ifdef WIN32
 #include <X11/Xwinsock.h>
 #endif
-#include "Xos.h"			/* for strings, fcntl, time */
+#include <X11/Xos.h>			/* for strings, fcntl, time */
 #include <errno.h>
 #include <stdio.h>
-#include "X.h"
+#include <X11/X.h>
 #include "misc.h"
 
 #ifdef __UNIXOS2__
@@ -99,6 +97,29 @@ static unsigned long startTimeInMillis;
 #include "dpmsproc.h"
 #endif
 
+#if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_WAKEUP)
+
+static unsigned long startTimeInMillis;
+
+#endif
+
+#ifdef WIN32
+/* Error codes from windows sockets differ from fileio error codes  */
+#undef EINTR
+#define EINTR WSAEINTR
+#undef EINVAL
+#define EINVAL WSAEINVAL
+#undef EBADF
+#define EBADF WSAENOTSOCK
+/* Windows select does not set errno. Use GetErrno as wrapper for 
+   WSAGetLastError */
+#define GetErrno WSAGetLastError
+#else
+/* This is just a fallback to errno to hide the differences between unix and
+   Windows in the code */
+#define GetErrno() errno
+#endif
+
 /* modifications by raphael */
 int
 mffs(fd_mask mask)
@@ -117,7 +138,7 @@ mffs(fd_mask mask)
 
 #ifdef DPMSExtension
 #define DPMS_SERVER
-#include "dpms.h"
+#include <X11/extensions/dpms.h>
 #endif
 
 #ifdef XTESTEXT1
@@ -135,7 +156,7 @@ struct _OsTimerRec {
 };
 
 static void DoTimer(OsTimerPtr timer, CARD32 now, OsTimerPtr *prev);
-static OsTimerPtr timers;
+static OsTimerPtr timers = NULL;
 
 /*****************
  * WaitForSomething:
@@ -154,18 +175,12 @@ static OsTimerPtr timers;
  *     pClientsReady is an array to store ready client->index values into.
  *****************/
 
-static INT32 timeTilFrob = 0;		/* while screen saving */
-
 int
-WaitForSomething(pClientsReady)
-    int *pClientsReady;
+WaitForSomething(int *pClientsReady)
 {
     int i;
     struct timeval waittime, *wt;
     INT32 timeout = 0;
-#ifdef DPMSExtension
-    INT32 standbyTimeout = 0, suspendTimeout = 0, offTimeout = 0;
-#endif
     fd_set clientsReadable;
     fd_set clientsWritable;
     int curclient;
@@ -222,138 +237,17 @@ WaitForSomething(pClientsReady)
 	else
 	{
 #endif
-#ifdef DPMSExtension
-	if (ScreenSaverTime > 0 || DPMSEnabled || timers)
-#else
-	if (ScreenSaverTime > 0 || timers)
-#endif
-	    now = GetTimeInMillis();
-	wt = NULL;
+        wt = NULL;
 	if (timers)
-	{
-	    while (timers && (int) (timers->expires - now) <= 0)
-		DoTimer(timers, now, &timers);
-	    if (timers)
-	    {
-		timeout = timers->expires - now;
-		waittime.tv_sec = timeout / MILLI_PER_SECOND;
-		waittime.tv_usec = (timeout % MILLI_PER_SECOND) *
-		    (1000000 / MILLI_PER_SECOND);
-		wt = &waittime;
-	    }
-	}
-	if (ScreenSaverTime > 0
-#ifdef DPMSExtension
-	    || (DPMSEnabled &&
-	     (DPMSStandbyTime > 0 || DPMSSuspendTime > 0 || DPMSOffTime > 0))
-#endif
-	) {
-#ifdef DPMSExtension
-	    if (ScreenSaverTime > 0)
-#endif
-		timeout = (ScreenSaverTime -
-			   (now - lastDeviceEventTime.milliseconds));
-#ifdef DPMSExtension
-	    if (DPMSStandbyTime > 0)
-		standbyTimeout = (DPMSStandbyTime -
-				  (now - lastDeviceEventTime.milliseconds));
-	    if (DPMSSuspendTime > 0)
-		suspendTimeout = (DPMSSuspendTime -
-				  (now - lastDeviceEventTime.milliseconds));
-	    if (DPMSOffTime > 0)
-		offTimeout = (DPMSOffTime -
-			      (now - lastDeviceEventTime.milliseconds));
-#endif /* DPMSExtension */
-
-	    if (
-		timeout <= 0
-#ifdef DPMSExtension
-		 && ScreenSaverTime > 0
-#endif /* DPMSExtension */
-	    ) {
-		INT32 timeSinceSave;
-
-		timeSinceSave = -timeout;
-		if (timeSinceSave >= timeTilFrob && timeTilFrob >= 0)
-		{
-		    ResetOsBuffers(); /* not ideal, but better than nothing */
-		    SaveScreens(SCREEN_SAVER_ON, ScreenSaverActive);
-#ifdef DPMSExtension
-		    if (ScreenSaverInterval > 0 &&
-			DPMSPowerLevel == DPMSModeOn)
-#else
-		    if (ScreenSaverInterval)
-#endif /* DPMSExtension */
-			/* round up to the next ScreenSaverInterval */
-			timeTilFrob = ScreenSaverInterval *
-				((timeSinceSave + ScreenSaverInterval) /
-					ScreenSaverInterval);
-		    else
-			timeTilFrob = -1;
-		}
-		timeout = timeTilFrob - timeSinceSave;
-	    }
-	    else
-	    {
-		if (ScreenSaverTime > 0 && timeout > ScreenSaverTime)
-		    timeout = ScreenSaverTime;
-		timeTilFrob = 0;
-	    }
-#ifdef DPMSExtension
-	    if (DPMSEnabled)
-	    {
-		if (standbyTimeout > 0 
-		    && (timeout <= 0 || timeout > standbyTimeout))
-		    timeout = standbyTimeout;
-		if (suspendTimeout > 0 
-		    && (timeout <= 0 || timeout > suspendTimeout))
-		    timeout = suspendTimeout;
-		if (offTimeout > 0 
-		    && (timeout <= 0 || timeout > offTimeout))
-		    timeout = offTimeout;
-	    }
-#endif
-	    if (timeout > 0 && (!wt || timeout < (int) (timers->expires - now)))
-	    {
-		waittime.tv_sec = timeout / MILLI_PER_SECOND;
-		waittime.tv_usec = (timeout % MILLI_PER_SECOND) *
-					(1000000 / MILLI_PER_SECOND);
-		wt = &waittime;
-	    }
-#ifdef DPMSExtension
-	    /* don't bother unless it's switched on */
-	    if (DPMSEnabled) {
-		/*
-		 * If this mode's enabled, and if the time's come
-		 * and if we're still at a lesser mode, do it now.
-		 */
-		if (DPMSStandbyTime > 0) {
-		    if (standbyTimeout <= 0) {
-			if (DPMSPowerLevel < DPMSModeStandby) {
-			    DPMSSet(DPMSModeStandby);
-			}
-		    }
-		}
-		/*
-		 * and ditto.  Note that since these modes can have the
-		 * same timeouts, they can happen at the same time.
-		 */
-		if (DPMSSuspendTime > 0) {
-		    if (suspendTimeout <= 0) {
-			if (DPMSPowerLevel < DPMSModeSuspend) {
-			    DPMSSet(DPMSModeSuspend);
-			}
-		    }
-		}
-		if (DPMSOffTime > 0) {
-		    if (offTimeout <= 0) {
-			if (DPMSPowerLevel < DPMSModeOff) {
-			    DPMSSet(DPMSModeOff);
-			}
-		    }
-		}
-           }
-#endif
+        {
+            now = GetTimeInMillis();
+	    timeout = timers->expires - now;
+            if (timeout < 0)
+                timeout = 0;
+	    waittime.tv_sec = timeout / MILLI_PER_SECOND;
+	    waittime.tv_usec = (timeout % MILLI_PER_SECOND) *
+		               (1000000 / MILLI_PER_SECOND);
+	    wt = &waittime;
 	}
 	XFD_COPYSET(&AllSockets, &LastSelectMask);
 #ifdef SMART_SCHEDULE
@@ -376,10 +270,12 @@ WaitForSomething(pClientsReady)
         /*
          * If caller has marked the first element of pClientsReady[],
          * bail out of select after a short timeout. We need this to
-         * let agent check if any client is connected or session can
-         * be closed after a timeout. We don't want to consume much
-         * CPU and this hack should keep WaitForSomething() backward
-         * compatible with existing servers.
+         * let the NX agent remove the splash screen when the timeout
+         * is expired. A better option would be to use the existing
+         * screen-saver timeout but it can be modified by clients, so
+         * we would need a special handling. This hack is trivial and
+         * keeps WaitForSomething() backward compatible with the exis-
+         * ting servers.
          */
 
         if (pClientsReady[0] == -1)
@@ -407,20 +303,23 @@ WaitForSomething(pClientsReady)
             fprintf(stderr, "WaitForSomething: Milliseconds to next wakeup are %ld.\n",
                         timeoutInMillis);
 #endif
-            if (wt == NULL || (waittime.tv_sec * MILLI_PER_SECOND +
-                    waittime.tv_usec / MILLI_PER_SECOND) >
-                        timeoutInMillis)
+            if (wt == NULL || (wt -> tv_sec * MILLI_PER_SECOND +
+                    wt -> tv_usec / MILLI_PER_SECOND) > timeoutInMillis)
             {
-                waittime.tv_sec = timeoutInMillis / MILLI_PER_SECOND;
-                waittime.tv_usec = (timeoutInMillis * MILLI_PER_SECOND) %
-                                        (MILLI_PER_SECOND * 1000);
+                if ((waittime.tv_sec * MILLI_PER_SECOND +
+                        waittime.tv_usec / MILLI_PER_SECOND) > timeoutInMillis)
+                {
+                    waittime.tv_sec = timeoutInMillis / MILLI_PER_SECOND;
+                    waittime.tv_usec = (timeoutInMillis * MILLI_PER_SECOND) %
+                                            (MILLI_PER_SECOND * 1000);
+                    wt = &waittime;
+                }
 
 #if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_WAKEUP) && defined(NX_TRANS_DEBUG)
                 fprintf(stderr, "WaitForSomething: Next wakeup timeout set to %ld milliseconds.\n",
                             (waittime.tv_sec * MILLI_PER_SECOND) +
                                 (waittime.tv_usec / MILLI_PER_SECOND));
 #endif
-                wt = &waittime;
             }
 #if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_WAKEUP) && defined(NX_TRANS_DEBUG)
             else
@@ -439,7 +338,7 @@ WaitForSomething(pClientsReady)
 	{
 	    i = -1;
 
-            fprintf(stderr, "WaitForSomething: Value of dispatchException is true. It set i = -1.\n");
+            fprintf(stderr, "WaitForSomething: Value of dispatchException is true. Set i = -1.\n");
 	}
 #else
         if (dispatchException)
@@ -448,7 +347,17 @@ WaitForSomething(pClientsReady)
 	else if (AnyClientsWriteBlocked)
 	{
 #if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_DEBUG)
-            fprintf(stderr, "WaitForSomething: Executing select with LastSelectMask and clientsWritable.\n");
+            if (wt == NULL)
+            {
+                fprintf(stderr, "WaitForSomething: Executing select with LastSelectMask and "
+                            "clientsWritable and null timeout.\n");
+            }
+            else
+            {
+                fprintf(stderr, "WaitForSomething: Executing select with LastSelectMask, "
+                            "clientsWritable, %ld secs and %ld usecs.\n",
+                                wt -> tv_sec, wt -> tv_usec);
+            }
 #endif
 	    XFD_COPYSET(&ClientsWriteBlocked, &clientsWritable);
 	    i = Select (MaxClients, &LastSelectMask, &clientsWritable, NULL, wt);
@@ -456,7 +365,15 @@ WaitForSomething(pClientsReady)
 	else 
 	{
 #if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_DEBUG)
-            fprintf(stderr, "WaitForSomething: Executing select with only LastSelectMask.\n");
+            if (wt == NULL)
+            {
+                fprintf(stderr, "WaitForSomething: Executing select with LastSelectMask and null timeout.\n");
+            }
+            else
+            {
+                fprintf(stderr, "WaitForSomething: Executing select with LastSelectMask, %ld secs and %ld usecs.\n",
+                            wt -> tv_sec, wt -> tv_usec);
+            }
 #endif
 	    i = Select (MaxClients, &LastSelectMask, NULL, NULL, wt);
 	}
@@ -468,7 +385,7 @@ WaitForSomething(pClientsReady)
             fprintf(stderr, "WaitForSomething: Error is [%s].\n", strerror(errno));
         }
 #endif
-	selecterr = errno;
+	selecterr = GetErrno();
 	WakeupHandler(i, (pointer)&LastSelectMask);
 #ifdef XTESTEXT1
 	if (playback_on) {
@@ -516,7 +433,7 @@ WaitForSomething(pClientsReady)
 		{
 		    FatalError("WaitForSomething(): select: errno=%d\n",
 			selecterr);
-		}
+            }
 		else if (selecterr != EINTR)
 		{
 		    ErrorF("WaitForSomething(): select: errno=%d\n",
@@ -534,12 +451,6 @@ WaitForSomething(pClientsReady)
 		break;
 	    }
 #endif
-	    if (timers)
-	    {
-		now = GetTimeInMillis();
-		while (timers && (int) (timers->expires - now) <= 0)
-		    DoTimer(timers, now, &timers);
-	    }
 #if defined(NX_TRANS_SOCKET)
             if (*checkForInput[0] != *checkForInput[1])
             {
@@ -549,13 +460,43 @@ WaitForSomething(pClientsReady)
 		return 0;
             }
 #else
-            if (*checkForInput[0] != *checkForInput[1])
-                return 0;
+	    if (*checkForInput[0] != *checkForInput[1])
+		return 0;
 #endif
+
+	    if (timers)
+	    {
+                int expired = 0;
+		now = GetTimeInMillis();
+		if ((int) (timers->expires - now) <= 0)
+		    expired = 1;
+
+		while (timers && (int) (timers->expires - now) <= 0)
+		    DoTimer(timers, now, &timers);
+
+                if (expired)
+                    return 0;
+	    }
 	}
 	else
 	{
 	    fd_set tmp_set;
+
+	    if (*checkForInput[0] == *checkForInput[1]) {
+	        if (timers)
+	        {
+                    int expired = 0;
+		    now = GetTimeInMillis();
+		    if ((int) (timers->expires - now) <= 0)
+		        expired = 1;
+
+		    while (timers && (int) (timers->expires - now) <= 0)
+		        DoTimer(timers, now, &timers);
+
+                    if (expired)
+                        return 0;
+	        }
+	    }
 #ifdef SMART_SCHEDULE
 	    if (someReady)
 		XFD_ORSET(&LastSelectMask, &ClientsWithInput, &LastSelectMask);
@@ -581,6 +522,23 @@ WaitForSomething(pClientsReady)
 #endif
 	    if (XFD_ANYSET (&devicesReadable) || XFD_ANYSET (&clientsReadable))
 		break;
+#ifdef WIN32
+	    /* Windows keyboard and mouse events are added to the input queue
+	       in Block- and WakupHandlers. There is no device to check if  
+	       data is ready. So check here if new input is available */
+#if defined(NX_TRANS_SOCKET)
+            if (*checkForInput[0] != *checkForInput[1])
+            {
+#if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_DEBUG)
+                fprintf(stderr, "WaitForSomething: Returning 0 because of (*checkForInput[0] != *checkForInput[1]).\n");
+#endif
+		return 0;
+            }
+#else
+	    if (*checkForInput[0] != *checkForInput[1])
+		return 0;
+#endif
+#endif
 	}
     }
 
@@ -608,7 +566,7 @@ WaitForSomething(pClientsReady)
 	    int client_priority, client_index;
 
 	    curclient = XFD_FD(&savedClientsReadable, i);
-	    client_index = ConnectionTranslation[curclient];
+	    client_index = GetConnectionTranslation(curclient);
 #endif
 #ifdef XSYNC
 		/*  We implement "strict" priorities.
@@ -659,8 +617,7 @@ WaitForSomething(pClientsReady)
 /*
  * This is not always a macro.
  */
-ANYSET(src)
-    FdMask	*src;
+ANYSET(FdMask *src)
 {
     int i;
 
@@ -685,12 +642,8 @@ DoTimer(OsTimerPtr timer, CARD32 now, OsTimerPtr *prev)
 }
 
 OsTimerPtr
-TimerSet(timer, flags, millis, func, arg)
-    register OsTimerPtr timer;
-    int flags;
-    CARD32 millis;
-    OsTimerCallback func;
-    pointer arg;
+TimerSet(OsTimerPtr timer, int flags, CARD32 millis, 
+    OsTimerCallback func, pointer arg)
 {
     register OsTimerPtr *prev;
     CARD32 now = GetTimeInMillis();
@@ -721,7 +674,7 @@ TimerSet(timer, flags, millis, func, arg)
     timer->expires = millis;
     timer->callback = func;
     timer->arg = arg;
-    if (millis <= now)
+    if ((int) (millis - now) <= 0)
     {
 	timer->next = NULL;
 	millis = (*timer->callback)(timer, now, timer->arg);
@@ -738,10 +691,9 @@ TimerSet(timer, flags, millis, func, arg)
 }
 
 Bool
-TimerForce(timer)
-    register OsTimerPtr timer;
+TimerForce(OsTimerPtr timer)
 {
-    register OsTimerPtr *prev;
+    OsTimerPtr *prev;
 
     for (prev = &timers; *prev; prev = &(*prev)->next)
     {
@@ -756,10 +708,9 @@ TimerForce(timer)
 
 
 void
-TimerCancel(timer)
-    register OsTimerPtr timer;
+TimerCancel(OsTimerPtr timer)
 {
-    register OsTimerPtr *prev;
+    OsTimerPtr *prev;
 
     if (!timer)
 	return;
@@ -774,8 +725,7 @@ TimerCancel(timer)
 }
 
 void
-TimerFree(timer)
-    register OsTimerPtr timer;
+TimerFree(OsTimerPtr timer)
 {
     if (!timer)
 	return;
@@ -784,16 +734,16 @@ TimerFree(timer)
 }
 
 void
-TimerCheck()
+TimerCheck(void)
 {
-    register CARD32 now = GetTimeInMillis();
+    CARD32 now = GetTimeInMillis();
 
     while (timers && (int) (timers->expires - now) <= 0)
 	DoTimer(timers, now, &timers);
 }
 
 void
-TimerInit()
+TimerInit(void)
 {
     OsTimerPtr timer;
 
@@ -803,3 +753,130 @@ TimerInit()
 	xfree(timer);
     }
 }
+
+static CARD32
+ScreenSaverTimeoutExpire(OsTimerPtr timer,CARD32 now,pointer arg)
+{
+    INT32 timeout = now - lastDeviceEventTime.milliseconds;
+
+    if (timeout < ScreenSaverTime) {
+        return ScreenSaverTime - timeout;
+    }
+
+    ResetOsBuffers(); /* not ideal, but better than nothing */
+    SaveScreens(SCREEN_SAVER_ON, ScreenSaverActive);
+
+#ifdef DPMSExtension
+    if (ScreenSaverInterval > 0 && DPMSPowerLevel == DPMSModeOn)
+#else
+    if (ScreenSaverInterval > 0)
+#endif /* DPMSExtension */
+        return ScreenSaverInterval;
+
+    return 0;
+}
+
+static OsTimerPtr ScreenSaverTimer = NULL;
+
+void
+FreeScreenSaverTimer(void)
+{
+    if (ScreenSaverTimer) {
+	TimerFree(ScreenSaverTimer);
+	ScreenSaverTimer = NULL;
+    }
+}
+
+void
+SetScreenSaverTimer(void)
+{
+    if (ScreenSaverTime > 0) {
+       ScreenSaverTimer = TimerSet(ScreenSaverTimer, 0, ScreenSaverTime,
+                                   ScreenSaverTimeoutExpire, NULL);
+    } else if (ScreenSaverTimer) {
+       FreeScreenSaverTimer();
+    }
+}
+
+#ifdef DPMSExtension
+
+static OsTimerPtr DPMSStandbyTimer = NULL;
+static OsTimerPtr DPMSSuspendTimer = NULL;
+static OsTimerPtr DPMSOffTimer = NULL;
+
+static CARD32
+DPMSStandbyTimerExpire(OsTimerPtr timer,CARD32 now,pointer arg)
+{
+    INT32 timeout = now - lastDeviceEventTime.milliseconds;
+
+    if (timeout < DPMSStandbyTime) {
+        return DPMSStandbyTime - timeout;
+    }
+    if (DPMSPowerLevel < DPMSModeStandby) {
+	if (DPMSEnabled)
+	    DPMSSet(DPMSModeStandby);
+    }
+    return DPMSStandbyTime;
+}
+
+static CARD32
+DPMSSuspendTimerExpire(OsTimerPtr timer,CARD32 now,pointer arg)
+{
+    INT32 timeout = now - lastDeviceEventTime.milliseconds;
+
+    if (timeout < DPMSSuspendTime) {
+        return DPMSSuspendTime - timeout;
+    }
+    if (DPMSPowerLevel < DPMSModeSuspend) {
+	if (DPMSEnabled)
+	    DPMSSet(DPMSModeSuspend);
+    }
+    return DPMSSuspendTime;
+}
+
+static CARD32
+DPMSOffTimerExpire(OsTimerPtr timer,CARD32 now,pointer arg)
+{
+    INT32 timeout = now - lastDeviceEventTime.milliseconds;
+
+    if (timeout < DPMSOffTime) {
+        return DPMSOffTime - timeout;
+    }
+    if (DPMSPowerLevel < DPMSModeOff) {
+	if (DPMSEnabled)
+	    DPMSSet(DPMSModeOff);
+    }
+    return DPMSOffTime;
+}
+
+void
+FreeDPMSTimers(void)
+{
+    if (DPMSStandbyTimer) {
+	TimerFree(DPMSStandbyTimer);
+	DPMSStandbyTimer = NULL;
+    }
+    if (DPMSSuspendTimer) {
+	TimerFree(DPMSSuspendTimer);
+	DPMSSuspendTimer = NULL;
+    }
+    if (DPMSOffTimer) {
+	TimerFree(DPMSOffTimer);
+	DPMSOffTimer = NULL;
+    }
+}
+
+void
+SetDPMSTimers(void)
+{
+    if (!DPMSEnabled)
+        return;
+
+    DPMSStandbyTimer = TimerSet(DPMSStandbyTimer, 0, DPMSStandbyTime,
+				DPMSStandbyTimerExpire, NULL);
+    DPMSSuspendTimer = TimerSet(DPMSSuspendTimer, 0, DPMSSuspendTime,
+				DPMSSuspendTimerExpire, NULL);
+    DPMSOffTimer = TimerSet(DPMSOffTimer, 0, DPMSOffTime,
+			    DPMSOffTimerExpire, NULL);
+}
+#endif
