@@ -1,3 +1,4 @@
+/* $XdotOrg: xc/programs/Xserver/os/utils.c,v 1.21 2005/11/08 06:33:30 jkj Exp $ */
 /* $Xorg: utils.c,v 1.5 2001/02/09 02:05:24 xorgcvs Exp $ */
 /*
 
@@ -49,24 +50,29 @@ OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
 OR PERFORMANCE OF THIS SOFTWARE.
 
 */
+/* $XFree86: xc/programs/Xserver/os/utils.c,v 3.96 2004/01/07 04:16:37 dawes Exp $ */
 
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001,2003 NoMachine, http://www.nomachine.com.           */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NX-X11, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
 /**************************************************************************/
 
+#ifdef HAVE_DIX_CONFIG_H
+#include <dix-config.h>
+#endif
+
 #ifdef __CYGWIN__
 #include <stdlib.h>
 #include <signal.h>
@@ -75,10 +81,11 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #if defined(WIN32) && !defined(__CYGWIN__)
 #include <X11/Xwinsock.h>
 #endif
-#include "Xos.h"
+#include <X11/Xos.h>
 #include <stdio.h>
 #include "misc.h"
-#include "X.h"
+#include <X11/X.h>
+#include <X11/Xtrans/Xtrans.h>
 #include "input.h"
 #include "dixfont.h"
 #include "osdep.h"
@@ -95,7 +102,9 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #undef _POSIX_SOURCE
 #endif
 #endif
+#ifndef WIN32
 #include <sys/wait.h>
+#endif
 #if !defined(SYSV) && !defined(WIN32) && !defined(Lynx) && !defined(QNX4)
 #include <sys/resource.h>
 #endif
@@ -124,41 +133,158 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #endif
 
 #ifdef XKB
-#include "XKBsrv.h"
+#include <X11/extensions/XKBsrv.h>
 #endif
 #ifdef XCSECURITY
 #define _SECURITY_SERVER
-#include "security.h"
+#include <X11/extensions/security.h>
 #endif
 
 #ifdef RENDER
 #include "picture.h"
 #endif
 
+#ifdef XPRINT
+#include "DiPrint.h"
+#endif
+
+Bool noTestExtensions;
+#ifdef BIGREQS
+Bool noBigReqExtension = FALSE;
+#endif
+#ifdef COMPOSITE
+/* COMPOSITE is disabled by default for now until the
+ * interface is stable */
+Bool noCompositeExtension = TRUE;
+#endif
+#ifdef DAMAGE
+Bool noDamageExtension = FALSE;
+#endif
+#ifdef DBE
+Bool noDbeExtension = FALSE;
+#endif
+#ifdef DPSEXT
+Bool noDPSExtension = FALSE;
+#endif
+#ifdef DPMSExtension
+Bool noDPMSExtension = FALSE;
+#endif
+#ifdef EVI
+Bool noEVIExtension = FALSE;
+#endif
+#ifdef FONTCACHE
+Bool noFontCacheExtension = FALSE;
+#endif
+#ifdef GLXEXT
+Bool noGlxExtension = FALSE;
+#endif
+#ifdef LBX
+Bool noLbxExtension = FALSE;
+#endif
+#ifdef SCREENSAVER
+Bool noScreenSaverExtension = FALSE;
+#endif
+#ifdef MITSHM
+Bool noMITShmExtension = FALSE;
+#endif
+#ifdef MITMISC
+Bool noMITMiscExtension = FALSE;
+#endif
+#ifdef MULTIBUFFER
+Bool noMultibufferExtension = FALSE;
+#endif
+#ifdef RANDR
+Bool noRRExtension = FALSE;
+#endif
+#ifdef RENDER
+Bool noRenderExtension = FALSE;
+#endif
+#ifdef SHAPE
+Bool noShapeExtension = FALSE;
+#endif
+#ifdef XCSECURITY
+Bool noSecurityExtension = FALSE;
+#endif
+#ifdef XSYNC
+Bool noSyncExtension = FALSE;
+#endif
+#ifdef TOGCUP
+Bool noXcupExtension = FALSE;
+#endif
+#ifdef RES
+Bool noResExtension = FALSE;
+#endif
+#ifdef XAPPGROUP
+Bool noXagExtension = FALSE;
+#endif
+#ifdef XCMISC
+Bool noXCMiscExtension = FALSE;
+#endif
+#ifdef XEVIE
+/* Xevie is disabled by default for now until the
+ * interface is stable */
+Bool noXevieExtension = TRUE;
+#endif
+#ifdef XF86BIGFONT
+Bool noXFree86BigfontExtension = FALSE;
+#endif
+#ifdef XFreeXDGA
+Bool noXFree86DGAExtension = FALSE;
+#endif
+#ifdef XF86DRI
+Bool noXFree86DRIExtension = FALSE;
+#endif
+#ifdef XF86MISC
+Bool noXFree86MiscExtension = FALSE;
+#endif
+#ifdef XF86VIDMODE
+Bool noXFree86VidModeExtension = FALSE;
+#endif
+#ifdef XFIXES
+Bool noXFixesExtension = FALSE;
+#endif
+/* |noXkbExtension| is defined in xc/programs/Xserver/xkb/xkbInit.c */
+#ifdef PANORAMIX
+/* Xinerama is disabled by default unless enabled via +xinerama */
+Bool noPanoramiXExtension = TRUE;
+#endif
+#ifdef XINPUT
+Bool noXInputExtension = FALSE;
+#endif
+#ifdef XIDLE
+Bool noXIdleExtension = FALSE;
+#endif
+#ifdef XV
+Bool noXvExtension = FALSE;
+#endif
+
 #define X_INCLUDE_NETDB_H
 #include <X11/Xos_r.h>
 
 #include <errno.h>
 
-#if defined(NX_TRANS_SOCKET)
-                                                                                                                                              
-#include NX_TRANS_INCLUDE
-                                                                                                                                              
+#ifdef NX_TRANS_SOCKET
+
+#include "NX.h"
+#include "NXvars.h"
+
+#endif
+
+#ifdef NX_TRANS_EXIT
+
+void (*OsVendorStartRedirectErrorFProc)() = NULL;
+void (*OsVendorEndRedirectErrorFProc)() = NULL;
+
 #endif
 
 Bool CoreDump;
-Bool noTestExtensions;
 
 #ifdef PANORAMIX
-Bool noPanoramiXExtension = TRUE;
 Bool PanoramiXVisibilityNotifySent = FALSE;
 Bool PanoramiXMapped = FALSE;
 Bool PanoramiXWindowExposureSent = FALSE;
 Bool PanoramiXOneExposeRequest = FALSE;
-#endif
-
-#ifdef DDXOSVERRORF
-void (*OsVendorVErrorFProc)(const char *, va_list args) = NULL;
+Bool PanoramiXExtensionDisabledHack = FALSE;
 #endif
 
 int auditTrailLevel = 1;
@@ -176,6 +302,10 @@ extern int SelectWaitTime;
 #endif
 #endif
 
+#if defined(SVR4) || defined(__linux__) || defined(CSRG_BASED)
+#define HAS_SAVED_IDS_AND_SETEUID
+#endif
+
 #ifdef MEMBUG
 #define MEM_FAIL_SCALE 100000
 long Memory_fail = 0;
@@ -188,6 +318,12 @@ int userdefinedfontpath = 0;
 
 char *dev_tty_from_init = NULL;		/* since we need to parse it anyway */
 
+extern char dispatchExceptionAtReset;
+
+/* Extension enable/disable in miinitext.c */
+extern Bool EnableDisableExtension(char *name, Bool enable);
+extern void EnableDisableExtensionError(char *name, Bool enable);
+
 OsSigHandlerPtr
 OsSignal(sig, handler)
     int sig;
@@ -257,12 +393,13 @@ static Bool nolock = FALSE;
  *      the lock file containing the PID.
  */
 void
-LockServer()
+LockServer(void)
 {
   char tmp[PATH_MAX], pid_str[12];
   int lfd, i, haslock, l_pid, t;
   char *tmppath = NULL;
   int len;
+  char port[20];
 
   if (nolock) return;
   /*
@@ -277,13 +414,14 @@ LockServer()
     FatalError("No TMP dir found\n");
 #endif
 
+  sprintf(port, "%d", atoi(display));
   len = strlen(LOCK_PREFIX) > strlen(LOCK_TMP_PREFIX) ? strlen(LOCK_PREFIX) :
 						strlen(LOCK_TMP_PREFIX);
-  len += strlen(tmppath) + strlen(display) + strlen(LOCK_SUFFIX) + 1;
+  len += strlen(tmppath) + strlen(port) + strlen(LOCK_SUFFIX) + 1;
   if (len > sizeof(LockFile))
-    FatalError("Display name `%s' is too long\n");
-  (void)sprintf(tmp, "%s" LOCK_TMP_PREFIX "%s" LOCK_SUFFIX, tmppath, display);
-  (void)sprintf(LockFile, "%s" LOCK_PREFIX "%s" LOCK_SUFFIX, tmppath, display);
+    FatalError("Display name `%s' is too long\n", port);
+  (void)sprintf(tmp, "%s" LOCK_TMP_PREFIX "%s" LOCK_SUFFIX, tmppath, port);
+  (void)sprintf(LockFile, "%s" LOCK_PREFIX "%s" LOCK_SUFFIX, tmppath, port);
 
   /*
    * Create a temporary file containing our PID.  Attempt three times
@@ -378,7 +516,7 @@ LockServer()
          */
         unlink(tmp);
 	FatalError("Server is already active for display %s\n%s %s\n%s\n",
-		   display, "\tIf this server is no longer running, remove",
+		   port, "\tIf this server is no longer running, remove",
 		   LockFile, "\tand start again.");
       }
     }
@@ -394,7 +532,7 @@ LockServer()
  *      Remove the server lock file.
  */
 void
-UnlockServer()
+UnlockServer(void)
 {
   if (nolock) return;
 
@@ -412,8 +550,7 @@ UnlockServer()
 
 /*ARGSUSED*/
 SIGVAL
-AutoResetServer (sig)
-    int sig;
+AutoResetServer (int sig)
 {
     int olderrno = errno;
 
@@ -433,11 +570,14 @@ AutoResetServer (sig)
 
 /*ARGSUSED*/
 SIGVAL
-GiveUp(sig)
-    int sig;
+GiveUp(int sig)
 {
     int olderrno = errno;
 
+#if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_TEST)
+    fprintf(stderr, "GiveUp: Called with signal [%d].\n", sig);
+#endif
+
     dispatchException |= DE_TERMINATE;
     isItTimeToYield = TRUE;
 #if defined(SYSV) && defined(X_NOT_POSIX)
@@ -447,45 +587,9 @@ GiveUp(sig)
     errno = olderrno;
 }
 
-#ifdef __GNUC__
-static void AbortServer() __attribute__((noreturn));
-#endif
-
-static void
-AbortServer()
-{
-#if defined(NX_TRANS_SOCKET) && defined(NX_TRANS_TEST)
-    fprintf(stderr, "AbortServer: Going to abort current server.\n");
-#endif
-    OsCleanup();
-    AbortDDX();
-    fflush(stderr);
-    if (CoreDump)
-	abort();
-#if defined(NX_TRANS_SOCKET)
-#ifdef NX_TRANS_TEST
-    fprintf(stderr, "AbortServer: Going to clean up NX resources and exit.\n");
-#endif
-#if defined(NX_TRANS_THREAD)
-    ExitMainThread(1);
-#else
-    NXExit(1);
-#endif
-#else
-    exit (1);
-#endif
-}
-
-void
-Error(str)
-    char *str;
-{
-    perror(str);
-}
-
 #ifndef DDXTIME
 CARD32
-GetTimeInMillis()
+GetTimeInMillis(void)
 {
     struct timeval  tp;
 
@@ -495,9 +599,7 @@ GetTimeInMillis()
 #endif
 
 void
-AdjustWaitForDelay (waitTime, newdelay)
-    pointer	    waitTime;
-    unsigned long   newdelay;
+AdjustWaitForDelay (pointer waitTime, unsigned long newdelay)
 {
     static struct timeval   delay_val;
     struct timeval	    **wt = (struct timeval **) waitTime;
@@ -520,7 +622,7 @@ AdjustWaitForDelay (waitTime, newdelay)
     }
 }
 
-void UseMsg()
+void UseMsg(void)
 {
 #if !defined(AIXrt) && !defined(AIX386)
     ErrorF("use: X [:<display>] [option]\n");
@@ -572,6 +674,8 @@ void UseMsg()
     ErrorF("nologo                 disable logo in screen saver\n");
 #endif
     ErrorF("-nolisten string       don't listen on protocol\n");
+    ErrorF("-noreset               don't reset after last client exists\n");
+    ErrorF("-reset                 reset after last client exists\n");
     ErrorF("-p #                   screen-saver pattern duration (minutes)\n");
     ErrorF("-pn                    accept failure to listen on all ports\n");
     ErrorF("-nopn                  reject failure to listen on all ports\n");
@@ -583,6 +687,9 @@ void UseMsg()
     ErrorF("-s #                   screen-saver timeout (minutes)\n");
 #ifdef XCSECURITY
     ErrorF("-sp file               security policy file\n");
+#endif
+#ifdef XPRINT
+    PrinterUseMsg();
 #endif
     ErrorF("-su                    disable any save under support\n");
     ErrorF("-t #                   mouse threshold (pixels)\n");
@@ -594,10 +701,17 @@ void UseMsg()
     ErrorF("-v                     screen-saver without video blanking\n");
     ErrorF("-wm                    WhenMapped default backing-store\n");
     ErrorF("-x string              loads named extension at init time \n");
+    ErrorF("-maxbigreqsize         set maximal bigrequest size \n");
 #ifdef PANORAMIX
     ErrorF("+xinerama              Enable XINERAMA extension\n");
     ErrorF("-xinerama              Disable XINERAMA extension\n");
 #endif
+#ifdef SMART_SCHEDULE
+    ErrorF("-dumbSched             Disable smart scheduling, enable old behavior\n");
+    ErrorF("-schedInterval int     Set scheduler interval in msec\n");
+#endif
+    ErrorF("+extension name        Enable extension\n");
+    ErrorF("-extension name        Disable extension\n");
 #ifdef XDMCP
     XdmcpUseMsg();
 #endif
@@ -615,8 +729,8 @@ void UseMsg()
  *  not contain a "/" and not start with a "-".
  *                                            --kvajk
  */
-int VerifyDisplayName( d )
-char *d;
+static int 
+VerifyDisplayName(const char *d)
 {
     if ( d == (char *)0 ) return( 0 );  /*  null  */
     if ( *d == '\0' ) return( 0 );  /*  empty  */
@@ -626,23 +740,33 @@ char *d;
     return( 1 );
 }
 
+/*
+ * This function is responsible for doing initalisation of any global
+ * variables at an very early point of server startup (even before
+ * |ProcessCommandLine()|. 
+ */
+void InitGlobals(void)
+{
+    ddxInitGlobals();
+}
+
+
 /*
  * This function parses the command line. Handles device-independent fields
  * and allows ddx to handle additional fields.  It is not allowed to modify
  * argc or any of the strings pointed to by argv.
  */
 void
-ProcessCommandLine ( argc, argv )
-int	argc;
-char	*argv[];
-
+ProcessCommandLine(int argc, char *argv[])
 {
     int i, skip;
 
     defaultKeyboardControl.autoRepeat = TRUE;
 
-#ifdef PART_NET
-	PartialNetwork = TRUE;
+#ifdef NO_PART_NET
+    PartialNetwork = FALSE;
+#else
+    PartialNetwork = TRUE;
 #endif
 
     for ( i = 1; i < argc; i++ )
@@ -660,7 +784,7 @@ char	*argv[];
             if( ! VerifyDisplayName( display ) ) {
                 ErrorF("Bad display name: %s\n", display);
                 UseMsg();
-                exit(1);
+		FatalError("Bad display name, exiting: %s\n", display);
             }
 	}
 	else if ( strcmp( argv[i], "-a") == 0)
@@ -853,17 +977,21 @@ char	*argv[];
 #endif
 	else if ( strcmp( argv[i], "-nolisten") == 0)
 	{
-            if(++i < argc)
-	        protNoListen = argv[i];
-	    else
+            if(++i < argc) {
+		if (_XSERVTransNoListen(argv[i])) 
+		    FatalError ("Failed to disable listen for %s transport",
+				argv[i]);
+	   } else
 		UseMsg();
 	}
 	else if ( strcmp( argv[i], "-noreset") == 0)
 	{
-	    extern char dispatchExceptionAtReset;
-
 	    dispatchExceptionAtReset = 0;
 	}
+	else if ( strcmp( argv[i], "-reset") == 0)
+	{
+	    dispatchExceptionAtReset = DE_RESET;
+	}
 	else if ( strcmp( argv[i], "-p") == 0)
 	{
 	    if(++i < argc)
@@ -899,8 +1027,6 @@ char	*argv[];
 	}
 	else if ( strcmp( argv[i], "-terminate") == 0)
 	{
-	    extern char dispatchExceptionAtReset;
-	    
 	    dispatchExceptionAtReset = DE_TERMINATE;
 	}
 	else if ( strcmp( argv[i], "-to") == 0)
@@ -920,6 +1046,24 @@ char	*argv[];
 	    defaultScreenSaverBlanking = DontPreferBlanking;
 	else if ( strcmp( argv[i], "-wm") == 0)
 	    defaultBackingStore = WhenMapped;
+        else if ( strcmp( argv[i], "-maxbigreqsize") == 0) {
+             if(++i < argc) {
+                 long reqSizeArg = atol(argv[i]);
+
+                 /* Request size > 128MB does not make much sense... */
+                 if( reqSizeArg > 0L && reqSizeArg < 128L ) {
+                     maxBigRequestSize = (reqSizeArg * 1048576L) - 1L;
+                 }
+                 else
+                 {
+                     UseMsg();
+                 }
+             }
+             else
+             {
+                 UseMsg();
+             }
+         }
 #ifdef PANORAMIX
 	else if ( strcmp( argv[i], "+xinerama") == 0){
 	    noPanoramiXExtension = FALSE;
@@ -927,6 +1071,9 @@ char	*argv[];
 	else if ( strcmp( argv[i], "-xinerama") == 0){
 	    noPanoramiXExtension = TRUE;
 	}
+	else if ( strcmp( argv[i], "-disablexineramaextension") == 0){
+	    PanoramiXExtensionDisabledHack = TRUE;
+	}
 #endif
 	else if ( strcmp( argv[i], "-x") == 0)
 	{
@@ -953,7 +1100,7 @@ char	*argv[];
 	}
 #endif
 #ifdef XPRINT
-	else if ((skip = XprintOptions(argc, argv, i)) != i)
+	else if ((skip = PrinterOptions(argc, argv, i)) != i)
 	{
 	    i = skip - 1;
 	}
@@ -1018,26 +1165,42 @@ char	*argv[];
 		UseMsg ();
 	}
 #endif
+	else if ( strcmp( argv[i], "+extension") == 0)
+	{
+	    if (++i < argc)
+	    {
+		if (!EnableDisableExtension(argv[i], TRUE))
+		    EnableDisableExtensionError(argv[i], TRUE);
+	    }
+	    else
+		UseMsg();
+	}
+	else if ( strcmp( argv[i], "-extension") == 0)
+	{
+	    if (++i < argc)
+	    {
+		if (!EnableDisableExtension(argv[i], FALSE))
+		    EnableDisableExtensionError(argv[i], FALSE);
+	    }
+	    else
+		UseMsg();
+	}
  	else
  	{
 	    ErrorF("Unrecognized option: %s\n", argv[i]);
 	    UseMsg();
-	    exit (1);
+	    FatalError("Unrecognized option: %s\n", argv[i]);
         }
     }
 }
 
 #ifdef COMMANDLINE_CHALLENGED_OPERATING_SYSTEMS
 static void
-InsertFileIntoCommandLine(resargc, resargv, prefix_argc, prefix_argv,
-			  filename, suffix_argc, suffix_argv)
-    int *resargc;
-    char ***resargv;
-    int prefix_argc;
-    char **prefix_argv;
-    char *filename;
-    int suffix_argc;
-    char **suffix_argv;
+InsertFileIntoCommandLine(
+    int *resargc, char ***resargv, 
+    int prefix_argc, char **prefix_argv,
+    char *filename, 
+    int suffix_argc, char **suffix_argv)
 {
     struct stat     st;
     FILE           *f;
@@ -1120,9 +1283,7 @@ InsertFileIntoCommandLine(resargc, resargv, prefix_argc, prefix_argv,
 
 
 void
-ExpandCommandLine(pargc, pargv)
-    int *pargc;
-    char ***pargv;
+ExpandCommandLine(int *pargc, char ***pargv)
 {
     int i;
 
@@ -1148,42 +1309,61 @@ ExpandCommandLine(pargc, pargv)
 /* Implement a simple-minded font authorization scheme.  The authorization
    name is "hp-hostname-1", the contents are simply the host name. */
 int
-set_font_authorizations(authorizations, authlen, client)
-char **authorizations;
-int *authlen;
-pointer client;
+set_font_authorizations(char **authorizations, int *authlen, pointer client)
 {
 #define AUTHORIZATION_NAME "hp-hostname-1"
 #if defined(TCPCONN) || defined(STREAMSCONN)
-    static char result[1024];
+    static char *result = NULL;
     static char *p = NULL;
 
     if (p == NULL)
     {
 	char hname[1024], *hnameptr;
+	unsigned int len;
+#if defined(IPv6) && defined(AF_INET6)
+	struct addrinfo hints, *ai = NULL;
+#else
 	struct hostent *host;
-	int len;
 #ifdef XTHREADS_NEEDS_BYNAMEPARAMS
 	_Xgethostbynameparams hparams;
+#endif
 #endif
 
 	gethostname(hname, 1024);
+#if defined(IPv6) && defined(AF_INET6)
+	bzero(&hints, sizeof(hints));
+	hints.ai_flags = AI_CANONNAME;
+	if (getaddrinfo(hname, NULL, &hints, &ai) == 0) {
+	    hnameptr = ai->ai_canonname;
+	} else {
+	    hnameptr = hname;
+	}
+#else
 	host = _XGethostbyname(hname, hparams);
 	if (host == NULL)
 	    hnameptr = hname;
 	else
 	    hnameptr = host->h_name;
+#endif
+
+	len = strlen(hnameptr) + 1;
+	result = xalloc(len + sizeof(AUTHORIZATION_NAME) + 4);
 
 	p = result;
         *p++ = sizeof(AUTHORIZATION_NAME) >> 8;
         *p++ = sizeof(AUTHORIZATION_NAME) & 0xff;
-        *p++ = (len = strlen(hnameptr) + 1) >> 8;
+        *p++ = (len) >> 8;
         *p++ = (len & 0xff);
 
 	memmove(p, AUTHORIZATION_NAME, sizeof(AUTHORIZATION_NAME));
 	p += sizeof(AUTHORIZATION_NAME);
 	memmove(p, hnameptr, len);
 	p += len;
+#if defined(IPv6) && defined(AF_INET6)
+	if (ai) {
+	    freeaddrinfo(ai);
+	}
+#endif
     }
     *authlen = p - result;
     *authorizations = result;
@@ -1208,8 +1388,7 @@ pointer client;
 #ifndef INTERNAL_MALLOC
 
 void * 
-Xalloc (amount)
-    unsigned long amount;
+Xalloc(unsigned long amount)
 {
     register pointer  ptr;
 	
@@ -1237,8 +1416,7 @@ Xalloc (amount)
  *****************/
 
 void *
-XNFalloc (amount)
-    unsigned long amount;
+XNFalloc(unsigned long amount)
 {
     register pointer ptr;
 
@@ -1261,8 +1439,7 @@ XNFalloc (amount)
  *****************/
 
 void *
-Xcalloc (amount)
-    unsigned long   amount;
+Xcalloc(unsigned long amount)
 {
     unsigned long   *ret;
 
@@ -1277,8 +1454,7 @@ Xcalloc (amount)
  *****************/
 
 void *
-XNFcalloc (amount)
-    unsigned long   amount;
+XNFcalloc(unsigned long amount)
 {
     unsigned long   *ret;
 
@@ -1295,9 +1471,7 @@ XNFcalloc (amount)
  *****************/
 
 void *
-Xrealloc (ptr, amount)
-    register pointer ptr;
-    unsigned long amount;
+Xrealloc(pointer ptr, unsigned long amount)
 {
 #ifdef MEMBUG
     if (!Must_have_memory && Memory_fail &&
@@ -1328,9 +1502,7 @@ Xrealloc (ptr, amount)
  *****************/
 
 void *
-XNFrealloc (ptr, amount)
-    register pointer ptr;
-    unsigned long amount;
+XNFrealloc(pointer ptr, unsigned long amount)
 {
     if (( ptr = (pointer)Xrealloc( ptr, amount ) ) == NULL)
     {
@@ -1346,15 +1518,14 @@ XNFrealloc (ptr, amount)
  *****************/    
 
 void
-Xfree(ptr)
-    register pointer ptr;
+Xfree(pointer ptr)
 {
     if (ptr)
 	free((char *)ptr); 
 }
 
 void
-OsInitAllocator ()
+OsInitAllocator (void)
 {
 #ifdef MEMBUG
     static int	been_here;
@@ -1397,110 +1568,6 @@ XNFstrdup(const char *s)
     return sd;
 }
 
-
-void
-AuditPrefix(f)
-    const char *f;
-{
-    time_t tm;
-    char *autime, *s;
-    if (*f != ' ')
-    {
-	time(&tm);
-	autime = ctime(&tm);
-	if ((s = strchr(autime, '\n')))
-	    *s = '\0';
-	if ((s = strrchr(argvGlobal[0], '/')))
-	    s++;
-	else
-	    s = argvGlobal[0];
-	ErrorF("AUDIT: %s: %d %s: ", autime, getpid(), s);
-    }
-}
-
-void
-AuditF(const char * f, ...)
-{
-    va_list args;
-
-    AuditPrefix(f);
-
-    va_start(args, f);
-    VErrorF(f, args);
-    va_end(args);
-}
-
-void
-FatalError(const char *f, ...)
-{
-    va_list args;
-    static Bool beenhere = FALSE;
-
-    if (beenhere)
-	ErrorF("\nFatalError re-entered, aborting\n");
-    else
-	ErrorF("\nFatal server error:\n");
-
-    va_start(args, f);
-    VErrorF(f, args);
-    va_end(args);
-    ErrorF("\n");
-#ifdef DDXOSFATALERROR
-    if (!beenhere)
-	OsVendorFatalError();
-#endif
-#ifdef ABORTONFATALERROR
-    abort();
-#endif
-    if (!beenhere) {
-	beenhere = TRUE;
-	AbortServer();
-    } else
-	abort();
-    /*NOTREACHED*/
-}
-
-void
-VErrorF(f, args)
-    const char *f;
-    va_list args;
-{
-#ifdef AIXV3
-    if (SyncOn)
-        sync();
-#else
-#ifdef DDXOSVERRORF
-    if (OsVendorVErrorFProc)
-	OsVendorVErrorFProc(f, args);
-    else
-	vfprintf(stderr, f, args);
-#else
-    vfprintf(stderr, f, args);
-#endif
-#endif /* AIXV3 */
-}
-
-void
-VFatalError(const char *msg, va_list args)
-{
-    VErrorF(msg, args);
-    ErrorF("\n");
-#ifdef DDXOSFATALERROR
-    OsVendorFatalError();
-#endif
-    AbortServer();
-    /*NOTREACHED*/
-}
-
-void
-ErrorF(const char * f, ...)
-{
-    va_list args;
-    va_start(args, f);
-    VErrorF(f, args);
-    va_end(args);
-}
-
 #ifdef SMART_SCHEDULE
 
 unsigned long	SmartScheduleIdleCount;
@@ -1516,12 +1583,21 @@ Bool		SmartScheduleTimerStopped;
 #define SMART_SCHEDULE_TIMER		ITIMER_REAL
 #endif
 
+#ifdef NX_TRANS_SOCKET
 void
 SmartScheduleStopTimer (void)
+#else
+static void
+SmartScheduleStopTimer (void)
+#endif
 {
 #ifdef SMART_SCHEDULE_POSSIBLE
     struct itimerval	timer;
-    
+
+    #ifdef NX_TRANS_TEST
+    fprintf(stderr, "SmartScheduleStopTimer: Stopping timer.\n");
+    #endif
+
     timer.it_interval.tv_sec = 0;
     timer.it_interval.tv_usec = 0;
     timer.it_value.tv_sec = 0;
@@ -1536,7 +1612,21 @@ SmartScheduleStartTimer (void)
 {
 #ifdef SMART_SCHEDULE_POSSIBLE
     struct itimerval	timer;
-    
+
+    #ifdef NX_TRANS_SOCKET
+
+    if (SmartScheduleDisable)
+    {
+      return FALSE;
+    }
+
+    #endif
+
+    #ifdef NX_TRANS_TEST
+    fprintf(stderr, "SmartScheduleStartTimer: Starting timer with [%ld] ms.\n",
+                SmartScheduleInterval);
+    #endif
+
     SmartScheduleTimerStopped = FALSE;
     timer.it_interval.tv_sec = 0;
     timer.it_interval.tv_usec = SmartScheduleInterval * 1000;
@@ -1548,12 +1638,18 @@ SmartScheduleStartTimer (void)
 }
 
 #ifdef SMART_SCHEDULE_POSSIBLE
-void
+static void
 SmartScheduleTimer (int sig)
 {
     int olderrno = errno;
 
     SmartScheduleTime += SmartScheduleInterval;
+
+    #ifdef NX_TRANS_TEST
+    fprintf(stderr, "SmartScheduleTimer: Got timer with time [%ld] ms.\n",
+                SmartScheduleTime);
+    #endif
+
     if (SmartScheduleIdle)
     {
 	SmartScheduleStopTimer ();
@@ -1571,6 +1667,10 @@ SmartScheduleInit (void)
     if (SmartScheduleDisable)
 	return TRUE;
     
+    #ifdef NX_TRANS_TEST
+    fprintf(stderr, "SmartScheduleInit: Initializing the smart scheduler.\n");
+    #endif
+
     bzero ((char *) &act, sizeof(struct sigaction));
 
     /* Set up the timer signal function */
@@ -1631,6 +1731,9 @@ OsBlockSignals (void)
 #endif
 #ifdef SIGTTOU
 	sigaddset (&set, SIGTTOU);
+#endif
+#ifdef SIGCHLD
+	sigaddset (&set, SIGCHLD);
 #endif
 	sigprocmask (SIG_BLOCK, &set, &PreviousSignalMask);
     }
@@ -1660,8 +1763,7 @@ OsReleaseSignals (void)
  */
 
 int
-System(command)
-    char *command;
+System(char *command)
 {
     int pid, p;
 #ifdef SIGCHLD
@@ -1680,12 +1782,19 @@ System(command)
     ErrorF("System: `%s'\n", command);
 #endif
 
+#ifdef NX_TRANS_EXIT
+    if (OsVendorStartRedirectErrorFProc != NULL) {
+        OsVendorStartRedirectErrorFProc();
+    }
+#endif
     switch (pid = fork()) {
     case -1:	/* error */
 	p = -1;
     case 0:	/* child */
-	setgid(getgid());
-	setuid(getuid());
+	if (setgid(getgid()) == -1)
+	    _exit(127);
+	if (setuid(getuid()) == -1)
+	    _exit(127);
 	execl("/bin/sh", "sh", "-c", command, (char *)NULL);
 	_exit(127);
     default:	/* parent */
@@ -1694,6 +1803,11 @@ System(command)
 	} while (p == -1 && errno == EINTR);
 	
     }
+#ifdef NX_TRANS_EXIT
+    if (OsVendorEndRedirectErrorFProc != NULL) {
+        OsVendorEndRedirectErrorFProc();
+    }
+#endif
 
 #ifdef SIGCHLD
     signal(SIGCHLD, csig);
@@ -1709,9 +1823,7 @@ static struct pid {
 } *pidlist;
 
 pointer
-Popen(command, type)
-    char *command;
-    char *type;
+Popen(char *command, char *type)
 {
     struct pid *cur;
     FILE *iop;
@@ -1731,6 +1843,11 @@ Popen(command, type)
 	return NULL;
     }
 
+#ifdef NX_TRANS_EXIT
+    if (OsVendorStartRedirectErrorFProc != NULL) {
+        OsVendorStartRedirectErrorFProc();
+    }
+#endif
     switch (pid = fork()) {
     case -1: 	/* error */
 	close(pdes[0]);
@@ -1738,8 +1855,10 @@ Popen(command, type)
 	xfree(cur);
 	return NULL;
     case 0:	/* child */
-	setgid(getgid());
-	setuid(getuid());
+	if (setgid(getgid()) == -1)
+	    _exit(127);
+	if (setuid(getuid()) == -1)
+	    _exit(127);
 	if (*type == 'r') {
 	    if (pdes[1] != 1) {
 		/* stdout */
@@ -1755,6 +1874,49 @@ Popen(command, type)
 	    }
 	    close(pdes[1]);
 	}
+
+        #ifdef NX_TRANS_SOCKET
+
+        /*
+         * Check if the child process should not
+         * use the parent's libraries.
+         */
+
+        if (_NXUnsetLibraryPath)
+        {
+          #ifndef __sun
+
+          unsetenv ("LD_LIBRARY_PATH");
+
+          #else
+
+          extern char **environ;
+
+          char **ep = environ;
+
+          ep = environ;
+
+          while (*ep)
+          {
+            if (!strncmp("LD_LIBRARY_PATH=", *ep, strlen("LD_LIBRARY_PATH=")))
+            {
+              break;
+            }
+
+            *ep++;
+          }
+
+          while (*ep)
+          {
+            *ep = *(ep + 1);
+            ep++;
+          }
+
+          #endif
+        }
+
+        #endif
+
 	execl("/bin/sh", "sh", "-c", command, (char *)NULL);
 	_exit(127);
     }
@@ -1783,9 +1945,102 @@ Popen(command, type)
     return iop;
 }
 
+/* fopen that drops privileges */
+pointer
+Fopen(char *file, char *type)
+{
+    FILE *iop;
+#ifndef HAS_SAVED_IDS_AND_SETEUID
+    struct pid *cur;
+    int pdes[2], pid;
+
+    if (file == NULL || type == NULL)
+	return NULL;
+
+    if ((*type != 'r' && *type != 'w') || type[1])
+	return NULL;
+
+    if ((cur = (struct pid *)xalloc(sizeof(struct pid))) == NULL)
+	return NULL;
+
+    if (pipe(pdes) < 0) {
+	xfree(cur);
+	return NULL;
+    }
+
+    switch (pid = fork()) {
+    case -1: 	/* error */
+	close(pdes[0]);
+	close(pdes[1]);
+	xfree(cur);
+	return NULL;
+    case 0:	/* child */
+	if (setgid(getgid()) == -1)
+	    _exit(127);
+	if (setuid(getuid()) == -1)
+	    _exit(127);
+	if (*type == 'r') {
+	    if (pdes[1] != 1) {
+		/* stdout */
+		dup2(pdes[1], 1);
+		close(pdes[1]);
+	    }
+	    close(pdes[0]);
+	} else {
+	    if (pdes[0] != 0) {
+		/* stdin */
+		dup2(pdes[0], 0);
+		close(pdes[0]);
+	    }
+	    close(pdes[1]);
+	}
+	execl("/bin/cat", "cat", file, (char *)NULL);
+	_exit(127);
+    }
+
+    /* Avoid EINTR during stdio calls */
+    OsBlockSignals ();
+    
+    /* parent */
+    if (*type == 'r') {
+	iop = fdopen(pdes[0], type);
+	close(pdes[1]);
+    } else {
+	iop = fdopen(pdes[1], type);
+	close(pdes[0]);
+    }
+
+    cur->fp = iop;
+    cur->pid = pid;
+    cur->next = pidlist;
+    pidlist = cur;
+
+#ifdef DEBUG
+    ErrorF("Popen: `%s', fp = %p\n", command, iop);
+#endif
+
+    return iop;
+#else
+    int ruid, euid;
+
+    ruid = getuid();
+    euid = geteuid();
+    
+    if (seteuid(ruid) == -1) {
+	    return NULL;
+    }
+    iop = fopen(file, type);
+
+    if (seteuid(euid) == -1) {
+	    fclose(iop);
+	    return NULL;
+    }
+    return iop;
+#endif /* HAS_SAVED_IDS_AND_SETEUID */
+}
+
 int
-Pclose(iop)
-    pointer iop;
+Pclose(pointer iop)
 {
     struct pid *cur, *last;
     int pstat;
@@ -1816,8 +2071,24 @@ Pclose(iop)
     /* allow EINTR again */
     OsReleaseSignals ();
     
+#ifdef NX_TRANS_EXIT
+    if (OsVendorEndRedirectErrorFProc != NULL) {
+        OsVendorEndRedirectErrorFProc();
+    }
+#endif
     return pid == -1 ? -1 : pstat;
 }
+
+int 
+Fclose(pointer iop)
+{
+#ifdef HAS_SAVED_IDS_AND_SETEUID
+    return fclose(iop);
+#else
+    return Pclose(iop);
+#endif
+}
+
 #endif /* !WIN32 && !__UNIXOS2__ */
 
 
@@ -1851,7 +2122,11 @@ Pclose(iop)
 
 /* Check args and env only if running setuid (euid == 0 && euid != uid) ? */
 #ifndef CHECK_EUID
+#ifndef WIN32
 #define CHECK_EUID 1
+#else
+#define CHECK_EUID 0
+#endif
 #endif
 
 /*
@@ -1883,16 +2158,24 @@ enum BadCode {
     InternalError
 };
 
+#if defined(VENDORSUPPORT)
+#define BUGADDRESS VENDORSUPPORT
+#elif defined(BUILDERADDR)
+#define BUGADDRESS BUILDERADDR
+#else
+#define BUGADDRESS "xorg@freedesktop.org"
+#endif
+
 #define ARGMSG \
     "\nIf the arguments used are valid, and have been rejected incorrectly\n" \
       "please send details of the arguments and why they are valid to\n" \
-      "XFree86@XFree86.org.  In the meantime, you can start the Xserver as\n" \
+      "%s.  In the meantime, you can start the Xserver as\n" \
       "the \"super user\" (root).\n"   
 
 #define ENVMSG \
     "\nIf the environment is valid, and have been rejected incorrectly\n" \
       "please send details of the environment and why it is valid to\n" \
-      "XFree86@XFree86.org.  In the meantime, you can start the Xserver as\n" \
+      "%s.  In the meantime, you can start the Xserver as\n" \
       "the \"super user\" (root).\n"
 
 void
@@ -1911,9 +2194,17 @@ CheckUserParameters(int argc, char **argv, char **envp)
     {
 	/* Check each argv[] */
 	for (i = 1; i < argc; i++) {
-	    if (strlen(argv[i]) > MAX_ARG_LENGTH) {
-		bad = ArgTooLong;
-		break;
+	    if (strcmp(argv[i], "-fp") == 0)
+	    {
+		i++; /* continue with next argument. skip the length check */
+		if (i >= argc)
+		    break;
+	    } else
+	    {
+		if (strlen(argv[i]) > MAX_ARG_LENGTH) {
+		    bad = ArgTooLong;
+		    break;
+		}
 	    }
 	    a = argv[i];
 	    while (*a) {
@@ -2000,20 +2291,20 @@ CheckUserParameters(int argc, char **argv, char **envp)
 	return;
     case UnsafeArg:
 	ErrorF("Command line argument number %d is unsafe\n", i);
-	ErrorF(ARGMSG);
+	ErrorF(ARGMSG, BUGADDRESS);
 	break;
     case ArgTooLong:
 	ErrorF("Command line argument number %d is too long\n", i);
-	ErrorF(ARGMSG);
+	ErrorF(ARGMSG, BUGADDRESS);
 	break;
     case UnprintableArg:
 	ErrorF("Command line argument number %d contains unprintable"
 		" characters\n", i);
-	ErrorF(ARGMSG);
+	ErrorF(ARGMSG, BUGADDRESS);
 	break;
     case EnvTooLong:
 	ErrorF("Environment variable `%s' is too long\n", e);
-	ErrorF(ENVMSG);
+	ErrorF(ENVMSG, BUGADDRESS);
 	break;
     case OutputIsPipe:
 	ErrorF("Stdout and/or stderr is a pipe\n");
@@ -2023,8 +2314,8 @@ CheckUserParameters(int argc, char **argv, char **envp)
 	break;
     default:
 	ErrorF("Unknown error\n");
-	ErrorF(ARGMSG);
-	ErrorF(ENVMSG);
+	ErrorF(ARGMSG, BUGADDRESS);
+	ErrorF(ENVMSG, BUGADDRESS);
 	break;
     }
     FatalError("X server aborted because of unsafe environment\n");
@@ -2043,7 +2334,7 @@ CheckUserParameters(int argc, char **argv, char **envp)
 #endif /* USE_PAM */
 
 void
-CheckUserAuthorization()
+CheckUserAuthorization(void)
 {
 #ifdef USE_PAM
     static struct pam_conv conv = {
@@ -2084,3 +2375,53 @@ CheckUserAuthorization()
     }
 #endif
 }
+
+#ifdef __SCO__
+#include <fcntl.h>
+
+static void
+lockit (int fd, short what)
+{
+  struct flock lck;
+
+  lck.l_whence = 0;
+  lck.l_start = 0;
+  lck.l_len = 1;
+  lck.l_type = what;
+
+  (void)fcntl (fd, F_SETLKW, &lck);
+}
+
+/* SCO OpenServer 5 lacks pread/pwrite. Emulate them. */
+ssize_t
+pread (int fd, void *buf, size_t nbytes, off_t offset)
+{
+  off_t saved;
+  ssize_t ret;
+
+  lockit (fd, F_RDLCK);
+  saved = lseek (fd, 0, SEEK_CUR);
+  lseek (fd, offset, SEEK_SET);
+  ret = read (fd, buf, nbytes);
+  lseek (fd, saved, SEEK_SET);
+  lockit (fd, F_UNLCK);
+
+  return ret;
+}
+
+ssize_t
+pwrite (int fd, const void *buf, size_t nbytes, off_t offset)
+{
+  off_t saved;
+  ssize_t ret;
+
+  lockit (fd, F_WRLCK);
+  saved = lseek (fd, 0, SEEK_CUR);
+  lseek (fd, offset, SEEK_SET);
+  ret = write (fd, buf, nbytes);
+  lseek (fd, saved, SEEK_SET);
+  lockit (fd, F_UNLCK);
+
+  return ret;
+}
+#endif /* __SCO__ */
